"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
# Generated by Django 3.1.4 on 2021-03-03 07:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [("io_storages", "0001_initial"), ("io_storages", "0002_auto_20210302_1827")]

    initial = True

    dependencies = [
        ("projects", "0061_delete_storage"),
        ("tasks", "__first__"),
    ]

    operations = [
        migrations.CreateModel(
            name="AzureBlobStorageMixin",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "container",
                    models.TextField(
                        blank=True,
                        help_text="Azure blob container",
                        null=True,
                        verbose_name="container",
                    ),
                ),
                (
                    "prefix",
                    models.TextField(
                        blank=True,
                        help_text="Azure blob prefix name",
                        null=True,
                        verbose_name="prefix",
                    ),
                ),
                (
                    "regex_filter",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage regex for filtering objects",
                        null=True,
                        verbose_name="regex_filter",
                    ),
                ),
                (
                    "use_blob_urls",
                    models.BooleanField(
                        default=False,
                        help_text="Interpret objects as BLOBs and generate URLs",
                        verbose_name="use_blob_urls",
                    ),
                ),
                (
                    "account_name",
                    models.TextField(
                        blank=True,
                        help_text="Azure Blob account name",
                        null=True,
                        verbose_name="account_name",
                    ),
                ),
                (
                    "account_key",
                    models.TextField(
                        blank=True,
                        help_text="Azure Blob account key",
                        null=True,
                        verbose_name="account_key",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="GCSStorageMixin",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "bucket",
                    models.TextField(
                        blank=True, help_text="GCS bucket name", null=True, verbose_name="bucket"
                    ),
                ),
                (
                    "prefix",
                    models.TextField(
                        blank=True, help_text="GCS bucket prefix", null=True, verbose_name="prefix"
                    ),
                ),
                (
                    "regex_filter",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage regex for filtering objects",
                        null=True,
                        verbose_name="regex_filter",
                    ),
                ),
                (
                    "use_blob_urls",
                    models.BooleanField(
                        default=False,
                        help_text="Interpret objects as BLOBs and generate URLs",
                        verbose_name="use_blob_urls",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="RedisStorageMixin",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "path",
                    models.TextField(
                        blank=True,
                        help_text="Storage prefix (optional)",
                        null=True,
                        verbose_name="path",
                    ),
                ),
                (
                    "host",
                    models.TextField(
                        blank=True,
                        help_text="Server Host IP (optional)",
                        null=True,
                        verbose_name="host",
                    ),
                ),
                (
                    "port",
                    models.TextField(
                        blank=True,
                        help_text="Server Port (optional)",
                        null=True,
                        verbose_name="port",
                    ),
                ),
                (
                    "password",
                    models.TextField(
                        blank=True,
                        help_text="Server Password (optional)",
                        null=True,
                        verbose_name="password",
                    ),
                ),
                (
                    "regex_filter",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage regex for filtering objects",
                        null=True,
                        verbose_name="port",
                    ),
                ),
                (
                    "use_blob_urls",
                    models.BooleanField(
                        default=False,
                        help_text="Interpret objects as BLOBs and generate URLs",
                        verbose_name="use_blob_urls",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="S3ExportStorage",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "bucket",
                    models.TextField(
                        blank=True, help_text="S3 bucket name", null=True, verbose_name="bucket"
                    ),
                ),
                (
                    "prefix",
                    models.TextField(
                        blank=True, help_text="S3 bucket prefix", null=True, verbose_name="prefix"
                    ),
                ),
                (
                    "regex_filter",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage regex for filtering objects",
                        null=True,
                        verbose_name="regex_filter",
                    ),
                ),
                (
                    "use_blob_urls",
                    models.BooleanField(
                        default=False,
                        help_text="Interpret objects as BLOBs and generate URLs",
                        verbose_name="use_blob_urls",
                    ),
                ),
                (
                    "aws_access_key_id",
                    models.TextField(
                        blank=True,
                        help_text="AWS_ACCESS_KEY_ID",
                        null=True,
                        verbose_name="aws_access_key_id",
                    ),
                ),
                (
                    "aws_secret_access_key",
                    models.TextField(
                        blank=True,
                        help_text="AWS_SECRET_ACCESS_KEY",
                        null=True,
                        verbose_name="aws_secret_access_key",
                    ),
                ),
                (
                    "aws_session_token",
                    models.TextField(
                        blank=True,
                        help_text="AWS_SESSION_TOKEN",
                        null=True,
                        verbose_name="aws_session_token",
                    ),
                ),
                (
                    "region_name",
                    models.TextField(
                        blank=True, help_text="AWS Region", null=True, verbose_name="region_name"
                    ),
                ),
                (
                    "s3_endpoint",
                    models.TextField(
                        blank=True, help_text="S3 Endpoint", null=True, verbose_name="s3_endpoint"
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_s3exportstorages",
                        to="projects.project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="S3ImportStorage",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "bucket",
                    models.TextField(
                        blank=True, help_text="S3 bucket name", null=True, verbose_name="bucket"
                    ),
                ),
                (
                    "prefix",
                    models.TextField(
                        blank=True, help_text="S3 bucket prefix", null=True, verbose_name="prefix"
                    ),
                ),
                (
                    "regex_filter",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage regex for filtering objects",
                        null=True,
                        verbose_name="regex_filter",
                    ),
                ),
                (
                    "use_blob_urls",
                    models.BooleanField(
                        default=False,
                        help_text="Interpret objects as BLOBs and generate URLs",
                        verbose_name="use_blob_urls",
                    ),
                ),
                (
                    "aws_access_key_id",
                    models.TextField(
                        blank=True,
                        help_text="AWS_ACCESS_KEY_ID",
                        null=True,
                        verbose_name="aws_access_key_id",
                    ),
                ),
                (
                    "aws_secret_access_key",
                    models.TextField(
                        blank=True,
                        help_text="AWS_SECRET_ACCESS_KEY",
                        null=True,
                        verbose_name="aws_secret_access_key",
                    ),
                ),
                (
                    "aws_session_token",
                    models.TextField(
                        blank=True,
                        help_text="AWS_SESSION_TOKEN",
                        null=True,
                        verbose_name="aws_session_token",
                    ),
                ),
                (
                    "region_name",
                    models.TextField(
                        blank=True, help_text="AWS Region", null=True, verbose_name="region_name"
                    ),
                ),
                (
                    "s3_endpoint",
                    models.TextField(
                        blank=True, help_text="S3 Endpoint", null=True, verbose_name="s3_endpoint"
                    ),
                ),
                (
                    "presign",
                    models.BooleanField(
                        default=True, help_text="Generate presigned URLs", verbose_name="presign"
                    ),
                ),
                (
                    "presign_ttl",
                    models.PositiveSmallIntegerField(
                        default=1,
                        help_text="Presigned URLs TTL (in minutes)",
                        verbose_name="presign_ttl",
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_s3importstorages",
                        to="projects.project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="AzureBlobExportStorage",
            fields=[
                (
                    "azureblobstoragemixin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="io_storages.azureblobstoragemixin",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_azureblobexportstorages",
                        to="projects.project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("io_storages.azureblobstoragemixin", models.Model),
        ),
        migrations.CreateModel(
            name="AzureBlobImportStorage",
            fields=[
                (
                    "azureblobstoragemixin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="io_storages.azureblobstoragemixin",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "presign",
                    models.BooleanField(
                        default=True, help_text="Generate presigned URLs", verbose_name="presign"
                    ),
                ),
                (
                    "presign_ttl",
                    models.PositiveSmallIntegerField(
                        default=1,
                        help_text="Presigned URLs TTL (in minutes)",
                        verbose_name="presign_ttl",
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_azureblobimportstorages",
                        to="projects.project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("io_storages.azureblobstoragemixin", models.Model),
        ),
        migrations.CreateModel(
            name="GCSExportStorage",
            fields=[
                (
                    "gcsstoragemixin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="io_storages.gcsstoragemixin",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_gcsexportstorages",
                        to="projects.project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("io_storages.gcsstoragemixin", models.Model),
        ),
        migrations.CreateModel(
            name="GCSImportStorage",
            fields=[
                (
                    "gcsstoragemixin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="io_storages.gcsstoragemixin",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "presign",
                    models.BooleanField(
                        default=True, help_text="Generate presigned URLs", verbose_name="presign"
                    ),
                ),
                (
                    "presign_ttl",
                    models.PositiveSmallIntegerField(
                        default=1,
                        help_text="Presigned URLs TTL (in minutes)",
                        verbose_name="presign_ttl",
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_gcsimportstorages",
                        to="projects.project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("io_storages.gcsstoragemixin", models.Model),
        ),
        migrations.CreateModel(
            name="RedisExportStorage",
            fields=[
                (
                    "redisstoragemixin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="io_storages.redisstoragemixin",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "db",
                    models.PositiveSmallIntegerField(
                        default=2, help_text="Server Database", verbose_name="db"
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_redisexportstorages",
                        to="projects.project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("io_storages.redisstoragemixin", models.Model),
        ),
        migrations.CreateModel(
            name="RedisImportStorage",
            fields=[
                (
                    "redisstoragemixin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="io_storages.redisstoragemixin",
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "db",
                    models.PositiveSmallIntegerField(
                        default=1, help_text="Server Database", verbose_name="db"
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_redisimportstorages",
                        to="projects.project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("io_storages.redisstoragemixin", models.Model),
        ),
        migrations.CreateModel(
            name="S3ImportStorageLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("key", models.TextField(help_text="External link key", verbose_name="key")),
                (
                    "object_exists",
                    models.BooleanField(
                        default=True,
                        help_text="Whether object under external link still exists",
                        verbose_name="object exists",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "storage",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="io_storages.s3importstorage",
                    ),
                ),
                (
                    "task",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_s3importstoragelink",
                        to="tasks.task",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="S3ExportStorageLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "object_exists",
                    models.BooleanField(
                        default=True,
                        help_text="Whether object under external link still exists",
                        verbose_name="object exists",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "annotation",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_s3exportstoragelink",
                        to="tasks.taskcompletion",
                    ),
                ),
                (
                    "storage",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="io_storages.s3exportstorage",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="RedisImportStorageLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("key", models.TextField(help_text="External link key", verbose_name="key")),
                (
                    "object_exists",
                    models.BooleanField(
                        default=True,
                        help_text="Whether object under external link still exists",
                        verbose_name="object exists",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "task",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_redisimportstoragelink",
                        to="tasks.task",
                    ),
                ),
                (
                    "storage",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="io_storages.redisimportstorage",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="RedisExportStorageLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "object_exists",
                    models.BooleanField(
                        default=True,
                        help_text="Whether object under external link still exists",
                        verbose_name="object exists",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "annotation",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_redisexportstoragelink",
                        to="tasks.taskcompletion",
                    ),
                ),
                (
                    "storage",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="io_storages.redisexportstorage",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="GCSImportStorageLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("key", models.TextField(help_text="External link key", verbose_name="key")),
                (
                    "object_exists",
                    models.BooleanField(
                        default=True,
                        help_text="Whether object under external link still exists",
                        verbose_name="object exists",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "task",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_gcsimportstoragelink",
                        to="tasks.task",
                    ),
                ),
                (
                    "storage",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="io_storages.gcsimportstorage",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="GCSExportStorageLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "object_exists",
                    models.BooleanField(
                        default=True,
                        help_text="Whether object under external link still exists",
                        verbose_name="object exists",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "annotation",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_gcsexportstoragelink",
                        to="tasks.taskcompletion",
                    ),
                ),
                (
                    "storage",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="io_storages.gcsexportstorage",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="AzureBlobImportStorageLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("key", models.TextField(help_text="External link key", verbose_name="key")),
                (
                    "object_exists",
                    models.BooleanField(
                        default=True,
                        help_text="Whether object under external link still exists",
                        verbose_name="object exists",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "task",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_azureblobimportstoragelink",
                        to="tasks.task",
                    ),
                ),
                (
                    "storage",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="io_storages.azureblobimportstorage",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="AzureBlobExportStorageLink",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "object_exists",
                    models.BooleanField(
                        default=True,
                        help_text="Whether object under external link still exists",
                        verbose_name="object exists",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "annotation",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="io_storages_azureblobexportstoragelink",
                        to="tasks.taskcompletion",
                    ),
                ),
                (
                    "storage",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="links",
                        to="io_storages.azureblobexportstorage",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import data_export.api
from django.shortcuts import redirect
from django.urls import include, path

from . import api, views

app_name = "projects"

# reverse for projects:name
_urlpatterns = [
    path("", views.project_list, name="project-index"),
    path(
        "<int:pk>/settings/",
        views.project_settings,
        name="project-settings",
        kwargs={"sub_path": ""},
    ),
    path("<int:pk>/settings/<sub_path>", views.project_settings, name="project-settings-anything"),
    path(
        "upload-example/",
        views.upload_example_using_config,
        name="project-upload-example-using-config",
    ),
]

# reverse for projects:api:name
_api_urlpatterns = [
    # CRUD
    path("", api.ProjectListAPI.as_view(), name="project-list"),
    path("<int:pk>/", api.ProjectAPI.as_view(), name="project-detail"),
    # Get next task
    path("<int:pk>/next/", api.ProjectNextTaskAPI.as_view(), name="project-next"),
    # Validate label config in general
    path("validate/", api.LabelConfigValidateAPI.as_view(), name="label-config-validate"),
    # Validate label config for project
    path(
        "<int:pk>/validate",
        api.ProjectLabelConfigValidateAPI.as_view(),
        name="project-label-config-validate",
    ),
    # Project summary
    path("<int:pk>/summary/", api.ProjectSummaryAPI.as_view(), name="project-summary"),
    # Tasks list for the project: get and destroy
    path("<int:pk>/tasks/", api.TasksListAPI.as_view(), name="project-tasks-list"),
    # Generate sample task for this project
    path("<int:pk>/sample-task/", api.ProjectSampleTask.as_view(), name="project-sample-task"),
]

_api_urlpatterns_templates = [
    path("", api.TemplateListAPI.as_view(), name="template-list"),
]


urlpatterns = [
    path("projects/", include(_urlpatterns)),
    path("api/projects/", include((_api_urlpatterns, app_name), namespace="api")),
    path(
        "api/templates/", include((_api_urlpatterns_templates, app_name), namespace="api-templates")
    ),
]



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
from django.utils.decorators import method_decorator
from drf_yasg.utils import swagger_auto_schema
from io_storages.azure_blob.models import AzureBlobImportStorage, AzureBlobExportStorage
from io_storages.azure_blob.serializers import (
    AzureBlobImportStorageSerializer,
    AzureBlobExportStorageSerializer,
)
from io_storages.api import (
    ImportStorageListAPI,
    ImportStorageDetailAPI,
    ImportStorageSyncAPI,
    ExportStorageListAPI,
    ExportStorageDetailAPI,
    ImportStorageValidateAPI,
    ExportStorageValidateAPI,
    ImportStorageFormLayoutAPI,
    ExportStorageFormLayoutAPI,
)


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Get all import storage",
        operation_description="Get list of all Azure import storage connections.",
    ),
)
@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Create new storage",
        operation_description="Get new Azure import storage",
    ),
)
class AzureBlobImportStorageListAPI(ImportStorageListAPI):
    queryset = AzureBlobImportStorage.objects.all()
    serializer_class = AzureBlobImportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Get import storage",
        operation_description="Get a specific Azure import storage connection.",
    ),
)
@method_decorator(
    name="patch",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Update import storage",
        operation_description="Update a specific Azure import storage connection.",
    ),
)
@method_decorator(
    name="delete",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Delete import storage",
        operation_description="Delete a specific Azure import storage connection.",
    ),
)
class AzureBlobImportStorageDetailAPI(ImportStorageDetailAPI):
    queryset = AzureBlobImportStorage.objects.all()
    serializer_class = AzureBlobImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Sync import storage",
        operation_description="Sync tasks from an Azure import storage connection.",
    ),
)
class AzureBlobImportStorageSyncAPI(ImportStorageSyncAPI):
    serializer_class = AzureBlobImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Validate import storage",
        operation_description="Validate a specific Azure import storage connection.",
    ),
)
class AzureBlobImportStorageValidateAPI(ImportStorageValidateAPI):
    serializer_class = AzureBlobImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Validate export storage",
        operation_description="Validate a specific Azure export storage connection.",
    ),
)
class AzureBlobExportStorageValidateAPI(ExportStorageValidateAPI):
    serializer_class = AzureBlobExportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Get all export storage",
        operation_description="Get a list of all Azure export storage connections.",
    ),
)
@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Create export storage",
        operation_description="Create a new Azure export storage connection to store annotations.",
    ),
)
class AzureBlobExportStorageListAPI(ExportStorageListAPI):
    queryset = AzureBlobExportStorage.objects.all()
    serializer_class = AzureBlobExportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Get export storage",
        operation_description="Get a specific Azure export storage connection.",
    ),
)
@method_decorator(
    name="patch",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Update export storage",
        operation_description="Update a specific Azure export storage connection.",
    ),
)
@method_decorator(
    name="delete",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Azure: Delete export storage",
        operation_description="Delete a specific Azure export storage connection.",
    ),
)
class AzureBlobExportStorageDetailAPI(ExportStorageDetailAPI):
    queryset = AzureBlobExportStorage.objects.all()
    serializer_class = AzureBlobExportStorageSerializer


class AzureBlobImportStorageFormLayoutAPI(ImportStorageFormLayoutAPI):
    pass


class AzureBlobExportStorageFormLayoutAPI(ExportStorageFormLayoutAPI):
    pass



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import json
import logging
import os
from pathlib import Path
import re

from django.conf import settings
from django.db import models, transaction
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils.translation import gettext_lazy as _
from rest_framework.exceptions import ValidationError

from core.utils.params import get_env
from io_storages.base_models import (
    ExportStorage,
    ExportStorageLink,
    ImportStorage,
    ImportStorageLink,
)
from io_storages.serializers import StorageAnnotationSerializer
from tasks.models import Annotation

logger = logging.getLogger(__name__)
url_scheme = "https"


class LocalFilesMixin(models.Model):
    path = models.TextField(_("path"), null=True, blank=True, help_text="Local path")
    regex_filter = models.TextField(
        _("regex_filter"), null=True, blank=True, help_text="Regex for filtering objects"
    )
    use_blob_urls = models.BooleanField(
        _("use_blob_urls"), default=False, help_text="Interpret objects as BLOBs and generate URLs"
    )

    def validate_connection(self):
        path = Path(self.path)
        if not path.exists():
            raise ValidationError(f"Path {self.path} does not exist")
        if settings.LOCAL_FILES_SERVING_ENABLED is False:
            raise ValidationError(
                "Serving local files can be dangerous, so it's disabled by default. "
                "You can enable it with LOCAL_FILES_SERVING_ENABLED environment variable"
            )


class LocalFilesImportStorage(LocalFilesMixin, ImportStorage):
    def iterkeys(self):
        path = Path(self.path)
        regex = re.compile(str(self.regex_filter)) if self.regex_filter else None
        for file in path.rglob("*"):
            if file.is_file():
                key = file.name
                if regex and not regex.match(key):
                    logger.debug(key + " is skipped by regex filter")
                    continue
                yield str(file)

    def get_data(self, key):
        path = Path(key)
        if self.use_blob_urls:
            # include self-hosted links pointed to local resources via {settings.HOSTNAME}/data/local-files?d=<path/to/local/dir>
            document_root = Path(get_env("LOCAL_FILES_DOCUMENT_ROOT", default="/"))
            relative_path = str(path.relative_to(document_root))
            return {
                settings.DATA_UNDEFINED_NAME: f"{settings.HOSTNAME}/data/local-files/?d={relative_path}"
            }

        try:
            with open(path, encoding="utf8") as f:
                value = json.load(f)
        except (UnicodeDecodeError, json.decoder.JSONDecodeError):
            raise ValueError(
                f"Can't import JSON-formatted tasks from {key}. If you're trying to import binary objects, "
                f'perhaps you\'ve forgot to enable "Treat every bucket object as a source file" option?'
            )

        if not isinstance(value, dict):
            raise ValueError(
                f"Error on key {key}: For {self.__class__.__name__} your JSON file must be a dictionary with one task."
            )  # noqa
        return value

    def scan_and_create_links(self):
        return self._scan_and_create_links(LocalFilesImportStorageLink)


class LocalFilesExportStorage(ExportStorage, LocalFilesMixin):
    def save_annotation(self, annotation):
        logger.debug(
            f"Creating new object on {self.__class__.__name__} Storage {self} for annotation {annotation}"
        )
        ser_annotation = self._get_serialized_data(annotation)
        with transaction.atomic():
            # Create export storage link
            link = LocalFilesExportStorageLink.create(annotation, self)
            key = os.path.join(self.path, f"{link.key}.json")
            try:
                with open(key, mode="w") as f:
                    json.dump(ser_annotation, f, indent=2)
            except Exception as exc:
                logger.error(
                    f"Can't export annotation {annotation} to local storage {self}. Reason: {exc}",
                    exc_info=True,
                )


class LocalFilesImportStorageLink(ImportStorageLink):
    storage = models.ForeignKey(
        LocalFilesImportStorage, on_delete=models.CASCADE, related_name="links"
    )


class LocalFilesExportStorageLink(ExportStorageLink):
    storage = models.ForeignKey(
        LocalFilesExportStorage, on_delete=models.CASCADE, related_name="links"
    )


@receiver(post_save, sender=Annotation)
def export_annotation_to_local_files(sender, instance, **kwargs):
    project = instance.task.project
    if hasattr(project, "io_storages_localfilesexportstorages"):
        for storage in project.io_storages_localfilesexportstorages.all():
            logger.debug(f"Export {instance} to Local Storage {storage}")
            storage.save_annotation(instance)



# Generated by Django 3.1.4 on 2021-03-09 13:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projects", "0006_auto_20210308_1559"),
    ]

    operations = [
        migrations.RenameField(
            model_name="project",
            old_name="show_ground_truths_first",
            new_name="show_ground_truth_first",
        ),
        migrations.AlterField(
            model_name="project",
            name="show_ground_truth_first",
            field=models.BooleanField(default=True, verbose_name="show ground truth first"),
        ),
    ]



# Generated by Django 3.1.4 on 2021-03-10 20:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("organizations", "0001_squashed_0008_auto_20201005_1552"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="organization",
            name="default_role",
        ),
        migrations.RemoveField(
            model_name="organizationmember",
            name="role",
        ),
    ]



# Generated by Django 3.1.4 on 2021-03-05 20:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("data_import", "0001_initial"),
        ("tasks", "0001_squashed_0041_taskcompletionhistory_was_cancelled"),
    ]

    operations = [
        migrations.AlterField(
            model_name="task",
            name="file_upload",
            field=models.ForeignKey(
                blank=True,
                help_text="Uploaded file used as data source for this task",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="tasks",
                to="data_import.fileupload",
            ),
        ),
    ]



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import pytest
import yaml
import json
import io
import os
import time

from rest_framework.authtoken.models import Token
from rest_framework.test import APIClient
from operator import itemgetter


@pytest.fixture
@pytest.mark.django_db
def client_and_token(business_client):
    token = Token.objects.get(user=business_client.business.admin)
    client = APIClient()
    client.credentials(HTTP_AUTHORIZATION="Token " + token.key)
    client.team_id = business_client.team.id
    client.organization_pk = business_client.organization.pk
    return client, token


@pytest.mark.django_db
def pytest_generate_tests(metafunc):
    if "test_suite" in metafunc.fixturenames:
        with io.open(
            os.path.join(os.path.dirname(__file__), "test_data/full_steps.yml"), encoding="utf-8"
        ) as f:
            test_suites = yaml.load(f)
            metafunc.parametrize("test_name, test_suite", list(test_suites.items()))


@pytest.fixture(autouse=True)
def delete_projects_at_the_end(db, request):
    from projects.models import Project

    def delete_projects():
        p = Project.objects.all()
        print(f"Deleting {p.count()} projects")
        p.delete()

    request.addfinalizer(delete_projects)


@pytest.mark.integration_tests
@pytest.mark.django_db
def test_full_steps(client_and_token, test_name, test_suite):
    print(f"Start {test_name}")
    client, token = client_and_token

    project_config = test_suite["project_config"]
    project_config["team_id"] = client.team_id
    chosen_ml_backend = test_suite["ml_backend"]
    upload_tasks = test_suite["upload_tasks"]
    annotations = test_suite["annotations"]
    prediction_tasks = test_suite.get("prediction_tasks", upload_tasks)
    wait_model = test_suite.get("wait_model", 10)
    expected_predictions = test_suite["expected_predictions"]

    def get(url, code=200):
        r = client.get(url)
        assert r.status_code == code
        return r.json()

    def post(url, payload, code=(200, 201)):
        r = client.post(
            url,
            data=json.dumps(payload),
            content_type="application/json",
            headers={"Authorization": f"Token {token}"},
        )  # noqa
        assert r.status_code in code
        return r.json()

    def patch(url, payload, code=200):
        r = client.patch(
            url,
            data=json.dumps(payload),
            content_type="application/json",
            headers={"Authorization": f"Token {token}"},
        )  # noqa
        assert r.status_code == code
        return r.json()

    # create project
    project_id = post("/api/projects/", project_config)["id"]

    # activate ML backend
    connections_list = get(f"/api/projects/{project_id}/backends/connections")
    chosen_connection = None
    for connection in connections_list:
        print(f"Test connection {connection}...")
        if connection["ml_backend"]["name"] == chosen_ml_backend:
            chosen_connection = connection
            break
    assert chosen_connection, f"Connection to {chosen_ml_backend} not found"

    patch(
        f"/api/projects/{project_id}/",
        {"ml_backend_active_connection": chosen_connection["id"], "active_learning_enabled": True},
    )

    # upload data
    post(f"/api/projects/{project_id}/tasks/bulk/", upload_tasks)

    # get tasks list
    tasks_list = get(f"/api/projects/{project_id}/tasks/")

    tasks_list = sorted(tasks_list, key=itemgetter("id"))

    for task, annotation in zip(tasks_list, annotations):
        # make annotation
        post(f'/api/tasks/{task["id"]}/annotations/', annotation)
        time.sleep(1)

    # get predictions
    time.sleep(wait_model)
    predictions = post(f"/api/projects/{project_id}/predict", prediction_tasks)
    # assert len(predictions['model_version']) > 0, predictions
    for prediction, expected_prediction in zip(predictions["results"], expected_predictions):
        assert prediction["result"] == expected_prediction["result"]
        assert prediction["score"] > expected_prediction["score_above"]

    # delete project and check whether model is deleted too
    r = client.post(f"/projects/{project_id}/delete/", content_type="text/html; charset=utf-8")
    assert r.status_code == 302

    print("Trying to get final predictions after model removing fails...")
    # TODO: check why this works but should fail
    post(f"/api/projects/{project_id}/predict", prediction_tasks)



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import logging
import inspect
import os

from rest_framework import generics
from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.exceptions import NotFound
from drf_yasg import openapi as openapi
from drf_yasg.utils import swagger_auto_schema
from django.utils.decorators import method_decorator

from core.permissions import (
    BaseRulesPermission,
    IsBusiness,
    get_object_with_permissions,
    all_permissions,
)
from core.utils.common import get_object_with_check_and_log
from core.utils.io import read_yaml
from io_storages.serializers import ImportStorageSerializer, ExportStorageSerializer
from projects.models import Project

logger = logging.getLogger(__name__)


class StorageAPIBasePermission(BaseRulesPermission):
    perm = "projects.change_project"


class ImportStorageListAPI(generics.ListCreateAPIView):
    parser_classes = (JSONParser, FormParser, MultiPartParser)
    permission_required = all_permissions.projects_change
    serializer_class = ImportStorageSerializer

    def get_queryset(self):
        project_pk = self.request.query_params.get("project")
        project = get_object_with_check_and_log(self.request, Project, pk=project_pk)
        self.check_object_permissions(self.request, project)
        ImportStorageClass = self.serializer_class.Meta.model
        return ImportStorageClass.objects.filter(project_id=project.id)


class ImportStorageDetailAPI(generics.RetrieveUpdateDestroyAPIView):
    """RUD storage by pk specified in URL"""

    parser_classes = (JSONParser, FormParser, MultiPartParser)
    serializer_class = ImportStorageSerializer
    permission_required = all_permissions.projects_change

    @swagger_auto_schema(auto_schema=None)
    def put(self, request, *args, **kwargs):
        return super(ImportStorageDetailAPI, self).put(request, *args, **kwargs)


class ExportStorageListAPI(generics.ListCreateAPIView):
    parser_classes = (JSONParser, FormParser, MultiPartParser)
    permission_required = all_permissions.projects_change
    serializer_class = ExportStorageSerializer

    def get_queryset(self):
        project_pk = self.request.query_params.get("project")
        project = get_object_with_check_and_log(self.request, Project, pk=project_pk)
        self.check_object_permissions(self.request, project)
        ImportStorageClass = self.serializer_class.Meta.model
        return ImportStorageClass.objects.filter(project_id=project.id)


class ExportStorageDetailAPI(generics.RetrieveUpdateDestroyAPIView):
    """RUD storage by pk specified in URL"""

    parser_classes = (JSONParser, FormParser, MultiPartParser)
    serializer_class = ExportStorageSerializer
    permission_required = all_permissions.projects_change

    @swagger_auto_schema(auto_schema=None)
    def put(self, request, *args, **kwargs):
        return super(ExportStorageDetailAPI, self).put(request, *args, **kwargs)


class ImportStorageSyncAPI(generics.GenericAPIView):

    parser_classes = (JSONParser, FormParser, MultiPartParser)
    permission_required = all_permissions.projects_change
    serializer_class = ImportStorageSerializer

    def get_queryset(self):
        ImportStorageClass = self.serializer_class.Meta.model
        return ImportStorageClass.objects.all()

    def post(self, request, *args, **kwargs):
        storage = self.get_object()
        # check connectivity & access, raise an exception if not satisfied
        storage.validate_connection()
        storage.sync()
        storage.refresh_from_db()
        return Response(self.serializer_class(storage).data)


class StorageValidateAPI(generics.CreateAPIView):
    parser_classes = (JSONParser, FormParser, MultiPartParser)
    permission_classes = (IsBusiness, StorageAPIBasePermission)

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        storage = self.serializer_class.Meta.model(**serializer.validated_data)
        storage.validate_connection()
        return Response()


class StorageFormLayoutAPI(generics.RetrieveAPIView):

    parser_classes = (JSONParser, FormParser, MultiPartParser)
    permission_classes = (IsBusiness, StorageAPIBasePermission)
    swagger_schema = None
    storage_type = None

    @swagger_auto_schema(auto_schema=None)
    def get(self, request, *args, **kwargs):
        form_layout_file = os.path.join(
            os.path.dirname(inspect.getfile(self.__class__)), "form_layout.yml"
        )
        if not os.path.exists(form_layout_file):
            raise NotFound(f'"form_layout.yml" is not found for {self.__class__.__name__}')

        form_layout = read_yaml(form_layout_file)
        return Response(form_layout[self.storage_type])


class ImportStorageValidateAPI(StorageValidateAPI):
    serializer_class = ImportStorageSerializer


class ExportStorageValidateAPI(StorageValidateAPI):
    serializer_class = ExportStorageSerializer


class ImportStorageFormLayoutAPI(StorageFormLayoutAPI):
    storage_type = "ImportStorage"


class ExportStorageFormLayoutAPI(StorageFormLayoutAPI):
    storage_type = "ExportStorage"



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import logging
import os
import sys
import io
import requests
import calendar
import threading
import json
import platform

from datetime import datetime
from django.conf import settings

from uuid import uuid4
from .common import get_app_version, get_bool_env, get_client_ip
from .io import get_config_dir

logger = logging.getLogger(__name__)


class ContextLog(object):
    def __init__(self):
        self.collect_analytics = get_bool_env("collect_analytics", True)
        self.version = get_app_version()
        self.server_id = self._get_server_id()

    def _get_label_studio_env(self):
        env = {}
        for env_key, env_value in os.environ.items():
            if env_key.startswith("LABEL_STUDIO_"):
                env[env_key] = env_value
        return env

    def _get_server_id(self):
        user_id_file = os.path.join(get_config_dir(), "user_id")
        if not os.path.exists(user_id_file):
            user_id = str(uuid4())
            with io.open(user_id_file, mode="w", encoding="utf-8") as fout:
                fout.write(user_id)
        else:
            with io.open(user_id_file, encoding="utf-8") as f:
                user_id = f.read()
        return user_id

    def _is_docker(self):
        path = "/proc/self/cgroup"
        return (
            os.path.exists("/.dockerenv")
            or os.path.isfile(path)
            and any("docker" in line for line in open(path, encoding="utf-8"))
        )

    def _get_timestamp_now(self):
        return calendar.timegm(datetime.now().utctimetuple())

    def _prepare_json(self, payload, request):
        j = payload["json"]
        view_name = payload["view_name"]
        if view_name in ("tasks:api:task-annotations", "tasks:api-annotations:annotation-detail"):
            types = [r.get("type") for r in j.get("result", [])]
            payload["json"] = {"result": types, "lead_time": j.get("lead_time")}

    def _get_response_content(self, response):
        try:
            return json.loads(response.content)
        except:
            return

    def _prepare_response(self, payload):
        view_name = payload["view_name"]
        if (
            view_name
            in (
                "data_export:api-projects:project-export",
                "data_manager:api:view-tasks",
                "data_manager:data_manager.api.ProjectActionsAPI",
                "data_manager:data_manager.api.TaskAPI",
                "projects:api-templates:template-list",
                "data_import:api-projects:project-file-upload-list",
                "tasks:api:task-annotations",
                "tasks:api-annotations:annotation-detail",
            )
            and payload["status_code"] in (200, 201)
        ):
            payload["response"] = None

    def _exclude_endpoint(self, request):
        if request.resolver_match and request.resolver_match.view_name in [
            "django.views.static.serve",
            "data_import:data-upload",
        ]:
            return True
        if request.GET.get("interaction", None) == "timer":
            return True

    def send(self, request=None, response=None, body=None):
        if settings.DEBUG:
            try:
                payload = self.create_payload(request, response, body)
            except Exception as exc:
                logger.error(exc, exc_info=True)
            else:
                if get_bool_env("DEBUG_CONTEXTLOG", False):
                    logger.debug(json.dumps(payload, indent=2))
                pass
        else:
            # ignore specific events
            if not self.collect_analytics or self._exclude_endpoint(request):
                return
            thread = threading.Thread(target=self.send_job, args=(request, response, body))
            thread.start()

    @staticmethod
    def browser_exists(request):
        return (
            hasattr(request, "user_agent")
            and request.user_agent
            and hasattr(request.user_agent, "browser")
            and request.user_agent.browser
        )

    def create_payload(self, request, response, body):
        payload = {
            "url": request.build_absolute_uri(),
            "server_id": self._get_server_id(),
            "server_time": self._get_timestamp_now(),
            "session_id": request.session.get("uid", None),
            "client_ip": get_client_ip(request),
            "is_docker": self._is_docker(),
            "python": str(sys.version_info[0]) + "." + str(sys.version_info[1]),
            "env": self._get_label_studio_env(),
            "version": self.version,
            "view_name": request.resolver_match.view_name if request.resolver_match else None,
            "namespace": request.resolver_match.namespace if request.resolver_match else None,
            "scheme": request.scheme,
            "method": request.method,
            "values": request.GET.dict(),
            "json": body,
            "language": request.LANGUAGE_CODE,
            "content_type": request.content_type,
            "content_length": int(request.environ.get("CONTENT_LENGTH"))
            if request.environ.get("CONTENT_LENGTH")
            else None,
            "status_code": response.status_code,
            "response": self._get_response_content(response),
        }
        if self.browser_exists(request):
            payload.update(
                {
                    "is_mobile": request.user_agent.is_mobile,
                    "is_tablet": request.user_agent.is_tablet,
                    "is_touch_capable": request.user_agent.is_touch_capable,
                    "is_pc": request.user_agent.is_pc,
                    "is_bot": request.user_agent.is_bot,
                    "browser": request.user_agent.browser.family,
                    "browser_version": request.user_agent.browser.version_string,
                    "os": request.user_agent.os.family,
                    "platform_system": platform.system(),
                    "platform_release": platform.release(),
                    "os_version": request.user_agent.os.version_string,
                    "device": request.user_agent.device.family,
                }
            )
        self._prepare_json(payload, request)
        self._prepare_response(payload)
        return payload

    def send_job(self, request, response, body):
        try:
            payload = self.create_payload(request, response, body)
        except:
            pass
        else:
            try:
                url = "https://tele.labelstud.io"
                requests.post(url=url, json=payload)
            except:
                pass



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
from django.urls import path, include

from .s3.api import *
from .azure_blob.api import *
from .gcs.api import *
from .redis.api import *
from .localfiles.api import *
from .all_api import *

app_name = "storages"

# IO Storages CRUD
_api_urlpatterns = [
    # All storages
    path("", AllImportStorageListAPI.as_view(), name="storage-list"),
    path("export", AllExportStorageListAPI.as_view(), name="export-storage-list"),
    path("types", AllImportStorageTypesAPI.as_view(), name="storage-types"),
    path("export/types", AllExportStorageTypesAPI.as_view(), name="export-storage-types"),
    # Amazon S3
    path("s3", S3ImportStorageListAPI.as_view(), name="storage-s3-list"),
    path("s3/<int:pk>", S3ImportStorageDetailAPI.as_view(), name="storage-s3-detail"),
    path("s3/<int:pk>/sync", S3ImportStorageSyncAPI.as_view(), name="storage-s3-sync"),
    path("s3/validate", S3ImportStorageValidateAPI.as_view(), name="storage-s3-validate"),
    path("s3/form", S3ImportStorageFormLayoutAPI.as_view(), name="storage-s3-form"),
    path("export/s3", S3ExportStorageListAPI.as_view(), name="export-storage-s3-list"),
    path("export/s3/<int:pk>", S3ExportStorageDetailAPI.as_view(), name="export-storage-s3-detail"),
    path(
        "export/s3/validate",
        S3ExportStorageValidateAPI.as_view(),
        name="export-storage-s3-validate",
    ),
    path("export/s3/form", S3ExportStorageFormLayoutAPI.as_view(), name="export-storage-s3-form"),
    # Microsoft Azure
    path("azure", AzureBlobImportStorageListAPI.as_view(), name="storage-azure-list"),
    path("azure/<int:pk>", AzureBlobImportStorageDetailAPI.as_view(), name="storage-azure-detail"),
    path("azure/<int:pk>/sync", AzureBlobImportStorageSyncAPI.as_view(), name="storage-azure-sync"),
    path(
        "azure/validate", AzureBlobImportStorageValidateAPI.as_view(), name="storage-azure-validate"
    ),
    path("azure/form", AzureBlobImportStorageFormLayoutAPI.as_view(), name="storage-azure-form"),
    path("export/azure", AzureBlobExportStorageListAPI.as_view(), name="export-storage-azure-list"),
    path(
        "export/azure/<int:pk>",
        AzureBlobExportStorageDetailAPI.as_view(),
        name="export-storage-azure-detail",
    ),
    path(
        "export/azure/validate",
        AzureBlobExportStorageValidateAPI.as_view(),
        name="export-storage-azure-validate",
    ),
    path(
        "export/azure/form",
        AzureBlobExportStorageFormLayoutAPI.as_view(),
        name="export-storage-azure-form",
    ),
    # Google Cloud Storage
    path("gcs", GCSImportStorageListAPI.as_view(), name="storage-gcs-list"),
    path("gcs/<int:pk>", GCSImportStorageDetailAPI.as_view(), name="storage-gcs-detail"),
    path("gcs/<int:pk>/sync", GCSImportStorageSyncAPI.as_view(), name="storage-gcs-sync"),
    path("gcs/validate", GCSImportStorageValidateAPI.as_view(), name="storage-gcs-validate"),
    path("gcs/form", GCSImportStorageFormLayoutAPI.as_view(), name="storage-gcs-form"),
    path("export/gcs", GCSExportStorageListAPI.as_view(), name="export-storage-gcs-list"),
    path(
        "export/gcs/<int:pk>", GCSExportStorageDetailAPI.as_view(), name="export-storage-gcs-detail"
    ),
    path(
        "export/gcs/validate",
        GCSExportStorageValidateAPI.as_view(),
        name="export-storage-gcs-validate",
    ),
    path(
        "export/gcs/form", GCSExportStorageFormLayoutAPI.as_view(), name="export-storage-gcs-form"
    ),
    # Redis DB
    path("redis", RedisImportStorageListAPI.as_view(), name="storage-redis-list"),
    path("redis/<int:pk>", RedisImportStorageDetailAPI.as_view(), name="storage-redis-detail"),
    path("redis/<int:pk>/sync", RedisImportStorageSyncAPI.as_view(), name="storage-redis-sync"),
    path("redis/validate", RedisImportStorageValidateAPI.as_view(), name="storage-redis-validate"),
    path("redis/form", RedisImportStorageFormLayoutAPI.as_view(), name="storage-redis-form"),
    path("export/redis", RedisExportStorageListAPI.as_view(), name="export-storage-redis-list"),
    path(
        "export/redis/<int:pk>",
        RedisExportStorageDetailAPI.as_view(),
        name="export-storage-redis-detail",
    ),
    path(
        "export/redis/validate",
        RedisExportStorageValidateAPI.as_view(),
        name="export-storage-redis-validate",
    ),
    path(
        "export/redis/form",
        RedisExportStorageFormLayoutAPI.as_view(),
        name="export-storage-redis-form",
    ),
    # Local files
    path("localfiles", LocalFilesImportStorageListAPI.as_view(), name="storage-localfiles-list"),
    path(
        "localfiles/<int:pk>",
        LocalFilesImportStorageDetailAPI.as_view(),
        name="storage-localfiles-detail",
    ),
    path(
        "localfiles/<int:pk>/sync",
        LocalFilesImportStorageSyncAPI.as_view(),
        name="storage-localfiles-sync",
    ),
    path(
        "localfiles/validate",
        LocalFilesImportStorageValidateAPI.as_view(),
        name="storage-localfiles-validate",
    ),
    path(
        "localfiles/form",
        LocalFilesImportStorageFormLayoutAPI.as_view(),
        name="storage-localfiles-form",
    ),
    path(
        "export/localfiles",
        LocalFilesExportStorageListAPI.as_view(),
        name="export-storage-localfiles-list",
    ),
    path(
        "export/localfiles/<int:pk>",
        LocalFilesExportStorageDetailAPI.as_view(),
        name="export-storage-localfiles-detail",
    ),
    path(
        "export/localfiles/validate",
        LocalFilesExportStorageValidateAPI.as_view(),
        name="export-storage-localfiles-validate",
    ),
    path(
        "export/localfiles/form",
        LocalFilesExportStorageFormLayoutAPI.as_view(),
        name="export-storage-localfiles-form",
    ),
]

urlpatterns = [
    path("api/storages/", include((_api_urlpatterns, app_name), namespace="api")),
]



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import logging
import base64
import boto3

from botocore.exceptions import ClientError
from urllib.parse import urlparse
from core.utils.params import get_env


logger = logging.getLogger(__name__)


def get_client_and_resource(
    aws_access_key_id=None,
    aws_secret_access_key=None,
    aws_session_token=None,
    region_name=None,
    s3_endpoint=None,
):
    aws_access_key_id = aws_access_key_id or get_env("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = aws_secret_access_key or get_env("AWS_SECRET_ACCESS_KEY")
    aws_session_token = aws_session_token or get_env("AWS_SESSION_TOKEN")
    logger.debug(
        f"Create boto3 session with "
        f"access key id={aws_access_key_id}, "
        f'secret key={aws_secret_access_key[:4] + "..." if aws_secret_access_key else None}, '
        f"session token={aws_session_token}"
    )
    session = boto3.Session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        aws_session_token=aws_session_token,
    )
    settings = {"region_name": region_name or get_env("S3_region") or "us-east-1"}
    s3_endpoint = s3_endpoint or get_env("S3_ENDPOINT")
    if s3_endpoint:
        settings["endpoint_url"] = s3_endpoint
    client = session.client("s3", config=boto3.session.Config(signature_version="s3v4"), **settings)
    resource = session.resource(
        "s3", config=boto3.session.Config(signature_version="s3v4"), **settings
    )
    return client, resource


def resolve_s3_url(url, client, presign=True):
    r = urlparse(url, allow_fragments=False)
    bucket_name = r.netloc
    key = r.path.lstrip("/")

    # Return blob as base64 encoded string if presigned urls are disabled
    if not presign:
        object = client.get_object(Bucket=bucket_name, Key=key)
        content_type = object["ResponseMetadata"]["HTTPHeaders"]["content-type"]
        object_b64 = (
            "data:"
            + content_type
            + ";base64,"
            + base64.b64encode(object["Body"].read()).decode("utf-8")
        )
        return object_b64

    # Otherwise try to generate presigned url
    try:
        presigned_url = client.generate_presigned_url(
            ClientMethod="get_object", Params={"Bucket": bucket_name, "Key": key}
        )
    except ClientError as exc:
        logger.warning(f"Can't generate presigned URL for {url}. Reason: {exc}")
        return url
    else:
        logger.debug(
            "Presigned URL {presigned_url} generated for {url}".format(
                presigned_url=presigned_url, url=url
            )
        )
        return presigned_url



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
# Generated by Django 3.1.4 on 2021-03-03 07:37

import annoying.fields
import core.utils.common
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [
        ("projects", "0001_initial"),
        ("projects", "0002_auto_20190318_2010"),
        ("projects", "0003_auto_20190318_2011"),
        ("projects", "0004_auto_20190318_2012"),
        ("projects", "0005_auto_20190318_2014"),
        ("projects", "0006_auto_20190321_1457"),
        ("projects", "0007_auto_20190326_2059"),
        ("projects", "0008_auto_20190328_2028"),
        ("projects", "0009_auto_20190403_1517"),
        ("projects", "0010_mlbackendscheduledjob"),
        ("projects", "0011_auto_20190418_2056"),
        ("projects", "0012_auto_20190423_1445"),
        ("projects", "0012_auto_20190422_1253"),
        ("projects", "0013_merge_20190424_0748"),
        ("projects", "0014_auto_20190424_0748"),
        ("projects", "0015_auto_20190506_1532"),
        ("projects", "0016_projecttemplate_ml_backend"),
        ("projects", "0017_auto_20190507_1126"),
        ("projects", "0018_auto_20190507_1633"),
        ("projects", "0019_auto_20190507_2010"),
        ("projects", "0020_auto_20190508_1850"),
        ("projects", "0021_auto_20190509_2057"),
        ("projects", "0022_auto_20190510_1324"),
        ("projects", "0023_project_show_skip_button"),
        ("projects", "0024_project_enable_empty_completion"),
        ("projects", "0025_project_cluster_annotation"),
        ("projects", "0026_auto_20190624_1254"),
        ("projects", "0029_auto_20190702_0959"),
        ("projects", "0030_mlbackendtrainjob"),
        ("projects", "0031_project_show_completion_history"),
        ("projects", "0032_auto_20190708_1648"),
        ("projects", "0033_auto_20190712_0849"),
        ("projects", "0034_auto_20190726_1928"),
        ("projects", "0035_auto_20190805_1955"),
        ("projects", "0036_auto_20190805_2058"),
        ("projects", "0037_remove_project_active_learning_enabled"),
        ("projects", "0038_auto_20190921_0830"),
        ("projects", "0039_auto_20190923_1323"),
        ("projects", "0040_auto_20190930_0909"),
        ("projects", "0041_project_use_kappa"),
        ("projects", "0042_auto_20191114_1430"),
        ("projects", "0043_auto_20191126_1145"),
        ("projects", "0044_auto_20191206_1641"),
        ("projects", "0045_auto_20200302_1221"),
        ("projects", "0046_project_agreement_method"),
        ("projects", "0047_auto_20200313_1925"),
        ("projects", "0048_auto_20200316_1400"),
        ("projects", "0049_project_control_weights"),
        ("projects", "0050_auto_20200417_1019"),
        ("projects", "0051_project_team"),
        ("projects", "0052_projecttemplate_organization"),
        ("projects", "0053_auto_20200504_1324"),
        ("projects", "0054_project_result_count"),
        ("projects", "0055_storage"),
        ("projects", "0056_set_project_skip_onboarding"),
        ("projects", "0057_auto_20201015_1553"),
        ("projects", "0058_project_organization"),
        ("projects", "0059_remove_project_team"),
        ("projects", "0058_remove_projecttemplate_business"),
        ("projects", "0059_auto_20210122_1542"),
        ("projects", "0060_merge_20210126_1328"),
        ("projects", "0061_delete_storage"),
        ("projects", "0062_auto_20210217_2135"),
        ("projects", "0063_auto_20210222_1246"),
        ("projects", "0064_auto_20210223_0726"),
        ("projects", "0065_auto_20210223_2014"),
    ]

    initial = True

    dependencies = [
        ("organizations", "0008_auto_20201005_1552"),
        # ('tasks', '0038_delete_storagelink'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        # ('organizations', '0004_auto_20200501_1751'),
    ]

    operations = [
        migrations.CreateModel(
            name="MLBackend",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "url",
                    models.TextField(
                        blank=True, default="http://localhost:8999", null=True, verbose_name="url"
                    ),
                ),
                (
                    "name",
                    models.TextField(blank=True, default="default", null=True, verbose_name="name"),
                ),
                (
                    "title",
                    models.TextField(
                        blank=True, default="Default ML backend", null=True, verbose_name="title"
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[("IN", "Internal"), ("EX", "External")],
                        default="IN",
                        max_length=100,
                        null=True,
                        verbose_name="type",
                    ),
                ),
                (
                    "description",
                    models.TextField(blank=True, default="", null=True, verbose_name="description"),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
                (
                    "created_by",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="custom_models",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="custom models",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Project",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=1000, verbose_name="name")),
                (
                    "label_config",
                    models.TextField(blank=True, null=True, verbose_name="label config"),
                ),
                (
                    "expert_instruction",
                    models.TextField(
                        blank=True, default="", null=True, verbose_name="expert instruction"
                    ),
                ),
                (
                    "show_instruction",
                    models.BooleanField(default=False, verbose_name="show instruction"),
                ),
                ("skip_onboarding", models.BooleanField(default=False)),
                (
                    "active_learning_enabled",
                    models.BooleanField(default=True, verbose_name="active learning enabled"),
                ),
                (
                    "maximum_completions",
                    models.IntegerField(default=1, verbose_name="maximum completion number"),
                ),
                (
                    "model_version",
                    models.TextField(
                        blank=True, default="", null=True, verbose_name="model version"
                    ),
                ),
                (
                    "data_types",
                    models.JSONField(default=dict, null=True, verbose_name="data_types"),
                ),
                ("has_finished", models.BooleanField(default=False, verbose_name="has finished")),
                ("is_published", models.BooleanField(default=False, verbose_name="published")),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
                (
                    "created_by",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="created_projects",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="created by",
                    ),
                ),
                (
                    "ml_backend",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="projects",
                        to="projects.mlbackend",
                    ),
                ),
            ],
            options={
                "db_table": "project",
            },
        ),
        migrations.CreateModel(
            name="ProjectOnboardingSteps",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "code",
                    models.CharField(
                        choices=[
                            ("DU", "Upload your data"),
                            ("CF", "Configure settings"),
                            ("PB", "Publish project"),
                            ("IE", "Invite annotators"),
                        ],
                        max_length=2,
                        null=True,
                    ),
                ),
                ("title", models.CharField(max_length=1000, verbose_name="title")),
                ("description", models.TextField(verbose_name="descrition")),
                ("order", models.IntegerField(default=0)),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
            ],
            options={
                "ordering": ["order"],
            },
        ),
        migrations.CreateModel(
            name="ProjectStats",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("data", models.JSONField(default=dict)),
                (
                    "model_version",
                    models.TextField(
                        blank=True, default="", null=True, verbose_name="model version"
                    ),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="stats",
                        to="projects.project",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="ProjectTemplate",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=1000, verbose_name="title")),
                ("description", models.TextField(verbose_name="descrition")),
                ("cover_image_url", models.CharField(max_length=1000, verbose_name="cover image")),
                ("input_example", models.TextField(verbose_name="input example")),
                (
                    "input_example_json",
                    models.JSONField(default=list, verbose_name="input example json"),
                ),
                ("output_example", models.TextField(verbose_name="output example")),
                (
                    "output_example_json",
                    models.JSONField(default=list, verbose_name="output example json"),
                ),
                ("label_config", models.TextField(verbose_name="label config")),
                (
                    "expert_instruction",
                    models.TextField(default="", verbose_name="expert instruction"),
                ),
                ("tags", models.JSONField(default=list, verbose_name="tags")),
                ("task_data", models.JSONField(default=list, verbose_name="task data")),
                ("is_published", models.BooleanField(default=True, verbose_name="published")),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
            ],
        ),
        migrations.CreateModel(
            name="UploadedDataFile",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("file", models.FileField(upload_to="uploads/%Y/%m/%d")),
            ],
        ),
        migrations.CreateModel(
            name="ProjectOnboarding",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("finished", models.BooleanField(default=False)),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE, to="projects.project"
                    ),
                ),
                (
                    "step",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="po_through",
                        to="projects.projectonboardingsteps",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="project",
            name="onboarding_steps",
            field=models.ManyToManyField(
                related_name="projects",
                through="projects.ProjectOnboarding",
                to="projects.ProjectOnboardingSteps",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="template_used",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="projects",
                to="projects.projecttemplate",
                verbose_name="projects",
            ),
        ),
        migrations.AddField(
            model_name="projectstats",
            name="num_completions",
            field=models.IntegerField(default=0, verbose_name="num_completions"),
        ),
        migrations.AddField(
            model_name="projectstats",
            name="num_honeypots",
            field=models.IntegerField(default=0, verbose_name="num_honeypots"),
        ),
        migrations.AddField(
            model_name="projectstats",
            name="num_completions_fit_predictions",
            field=models.FloatField(default=0.0, verbose_name="num_completions_fit_predictions"),
        ),
        migrations.AddField(
            model_name="projectstats",
            name="num_honeypots_fit_predictions",
            field=models.FloatField(default=0.0, verbose_name="num_honeypots_fit_predictions"),
        ),
        migrations.RemoveField(
            model_name="project",
            name="ml_backend",
        ),
        migrations.CreateModel(
            name="MLBackendConnection",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("schema", models.JSONField(default=dict)),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                (
                    "ml_backend",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="project_connections",
                        to="projects.mlbackend",
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ml_backend_connections",
                        to="projects.project",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="project",
            name="ml_backend_active_connection",
            field=models.OneToOneField(
                help_text="The connection ID that identifies current ML backend.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="active_project",
                to="projects.mlbackendconnection",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="batch_size",
            field=models.IntegerField(
                default=100, help_text="Batch size for machine learning", verbose_name="batch_size"
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="interval",
            field=models.IntegerField(
                default=100,
                help_text="Interval of calculations for MLBackend",
                verbose_name="interval",
            ),
        ),
        migrations.CreateModel(
            name="MLBackendScheduledJob",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("job_id", models.CharField(max_length=128)),
                ("created_on", models.DateTimeField(auto_now_add=True)),
                (
                    "project",
                    models.OneToOneField(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="scheduled_job",
                        to="projects.project",
                    ),
                ),
            ],
        ),
        migrations.AlterField(
            model_name="project",
            name="active_learning_enabled",
            field=models.BooleanField(default=False, verbose_name="active learning enabled"),
        ),
        migrations.AlterField(
            model_name="project",
            name="active_learning_enabled",
            field=models.BooleanField(default=False, verbose_name="active learning enabled"),
        ),
        migrations.AlterField(
            model_name="mlbackend",
            name="created_by",
            field=models.ForeignKey(
                help_text="User ID who creates this model",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="custom_models",
                to=settings.AUTH_USER_MODEL,
                verbose_name="custom models",
            ),
        ),
        migrations.AlterField(
            model_name="mlbackend",
            name="description",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Model description",
                null=True,
                verbose_name="description",
            ),
        ),
        migrations.AlterField(
            model_name="mlbackend",
            name="name",
            field=models.TextField(
                blank=True,
                default="default",
                help_text="Model name",
                null=True,
                verbose_name="name",
            ),
        ),
        migrations.AlterField(
            model_name="mlbackend",
            name="title",
            field=models.TextField(
                blank=True,
                default="Default ML backend",
                help_text="Model title",
                null=True,
                verbose_name="title",
            ),
        ),
        migrations.AlterField(
            model_name="mlbackend",
            name="type",
            field=models.CharField(
                choices=[("IN", "Internal"), ("EX", "External")],
                default="IN",
                help_text="Backend type",
                max_length=100,
                null=True,
                verbose_name="type",
            ),
        ),
        migrations.AlterField(
            model_name="mlbackend",
            name="url",
            field=models.TextField(
                blank=True,
                default="http://localhost:8999",
                help_text="Model server URL",
                null=True,
                verbose_name="url",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="active_learning_enabled",
            field=models.BooleanField(
                default=False,
                help_text="Enable machine learning backend and active learning strategy for the task selection",
                verbose_name="active learning enabled",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="expert_instruction",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Expert instruction in HTML format",
                null=True,
                verbose_name="expert instruction",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="is_published",
            field=models.BooleanField(
                default=False,
                help_text="Project publishing flag for experts",
                verbose_name="published",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="label_config",
            field=models.TextField(
                blank=True,
                help_text='Label config in XML format. More about it in <a href="https://app.heartex.ai/static/docs/#/Business?id=label-config">documentation</a>',
                null=True,
                verbose_name="label config",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="maximum_completions",
            field=models.IntegerField(
                default=1,
                help_text="Maximum overlaps of expert completions for one task. If the completion number per task is equal or greater to this value, the task becomes finished (is_labeled=True)",
                verbose_name="maximum completion number",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="model_version",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Machine learning model version",
                null=True,
                verbose_name="model version",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="show_instruction",
            field=models.BooleanField(
                default=False,
                help_text="Show the instruction to the expert before he starts",
                verbose_name="show instruction",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="skip_onboarding",
            field=models.BooleanField(default=False, help_text="Skip onboarding steps"),
        ),
        migrations.AlterField(
            model_name="project",
            name="title",
            field=models.CharField(help_text="Project name", max_length=1000, verbose_name="name"),
        ),
        migrations.AddField(
            model_name="projecttemplate",
            name="ml_backend",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="project_templates",
                to="projects.mlbackend",
            ),
        ),
        migrations.AlterField(
            model_name="projectonboardingsteps",
            name="code",
            field=models.CharField(
                choices=[
                    ("DU", "Upload your data"),
                    ("CF", "Configure settings"),
                    ("PB", "Publish project"),
                    ("IE", "Invite collaborators"),
                ],
                max_length=2,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="title",
            field=models.CharField(
                help_text="Project name, lenth is from 3 to 50 chars",
                max_length=50,
                validators=[
                    django.core.validators.MinLengthValidator(3),
                    django.core.validators.MaxLengthValidator(50),
                ],
                verbose_name="name",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="show_skip_button",
            field=models.BooleanField(
                default=True,
                help_text="Show a skip button in interface and allow collaborators to skip the task",
                verbose_name="show skip button",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="enable_empty_completion",
            field=models.BooleanField(
                default=True,
                help_text="Allow submit empty completions",
                verbose_name="enable empty completion",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="cluster_annotation",
            field=models.BooleanField(
                default=False,
                help_text="If enabled, completions for all tasks in one cluster are propagated",
                verbose_name="cluster annotation",
            ),
        ),
        migrations.AlterField(
            model_name="projectonboardingsteps",
            name="description",
            field=models.TextField(verbose_name="description"),
        ),
        migrations.AlterUniqueTogether(
            name="project",
            unique_together={("title", "created_by")},
        ),
        migrations.AddField(
            model_name="project",
            name="min_completions_to_start_training",
            field=models.IntegerField(
                default=10,
                help_text="Minimum number of completed tasks after which training is started",
                verbose_name="min_completions_to_start_training",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="batch_size",
            field=models.IntegerField(
                default=1,
                help_text="How many predictions are sent to ML backend in one request",
                verbose_name="batch_size",
            ),
        ),
        migrations.RemoveField(
            model_name="project",
            name="interval",
        ),
        migrations.AddField(
            model_name="project",
            name="show_completion_history",
            field=models.BooleanField(
                default=False,
                help_text="Show completion history to collaborator",
                verbose_name="show completion history",
            ),
        ),
        migrations.CreateModel(
            name="MLBackendTrainJob",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("job_id", models.CharField(max_length=128)),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ml_backend_train_jobs",
                        to="projects.project",
                    ),
                ),
                (
                    "completed_tasks_count",
                    models.IntegerField(
                        default=0,
                        help_text="Number of completed tasks at the moment when this job is created",
                        verbose_name="completed_tasks_count",
                    ),
                ),
                (
                    "is_finished",
                    models.BooleanField(
                        default=False,
                        help_text="Whether this job is finished.",
                        verbose_name="is_finished",
                    ),
                ),
                (
                    "model_version",
                    models.TextField(
                        blank=True,
                        default="",
                        help_text="Model version returned when job is finished",
                        null=True,
                        verbose_name="model version",
                    ),
                ),
            ],
        ),
        migrations.AlterField(
            model_name="project",
            name="batch_size",
            field=models.IntegerField(
                default=100,
                help_text="How many predictions are sent to ML backend in one request",
                verbose_name="batch_size",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="show_collab_predictions",
            field=models.BooleanField(
                default=True,
                help_text="If set collaborator gets model predictions",
                verbose_name="show predictions to collaborator",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="label_config",
            field=models.TextField(
                blank=True,
                help_text="Label config in XML format. See more about it in documentation",
                null=True,
                verbose_name="label config",
            ),
        ),
        migrations.RemoveField(
            model_name="project",
            name="active_learning_enabled",
        ),
        migrations.AddField(
            model_name="project",
            name="show_ground_truths_first",
            field=models.BooleanField(default=True, verbose_name="show ground truths first"),
        ),
        migrations.AddField(
            model_name="project",
            name="sampling",
            field=models.CharField(
                choices=[
                    ("Sequential sampling", "Tasks are ordered by their IDs"),
                    ("Uniform sampling", "Tasks are chosen randomly"),
                    (
                        "Uncertainty sampling",
                        "Tasks are chosen according to model uncertainty scores (active learning mode)",
                    ),
                ],
                default="Uniform sampling",
                max_length=100,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="task_data_login",
            field=models.CharField(
                blank=True,
                help_text="Task data credentials: login",
                max_length=100,
                null=True,
                verbose_name="task_data_login",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="task_data_password",
            field=models.CharField(
                blank=True,
                help_text="Task data credentials: password",
                max_length=100,
                null=True,
                verbose_name="task_data_password",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="use_kappa",
            field=models.BooleanField(
                default=False,
                help_text="If categorical variables are used in labeling (e.g. choices), Cohen's Kappa statistic is computed to measure inter-rater reliability instead of basic agreement",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="metric_name",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Evaluation metric chosen for this project",
                null=True,
                verbose_name="metric_name",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="metric_params",
            field=models.JSONField(
                blank=True, default=dict, null=True, verbose_name="metric params"
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="ml_backend_active_connection",
            field=models.OneToOneField(
                blank=True,
                help_text="The connection ID that identifies current ML backend.",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="active_project",
                to="projects.mlbackendconnection",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="template_used",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="projects",
                to="projects.projecttemplate",
                verbose_name="Project templates",
            ),
        ),
        migrations.AlterField(
            model_name="projectonboardingsteps",
            name="code",
            field=models.CharField(
                choices=[
                    ("DU", "Import your data"),
                    ("CF", "Configure settings"),
                    ("PB", "Publish project"),
                    ("IE", "Invite collaborators"),
                ],
                max_length=2,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="metric_threshold",
            field=models.FloatField(
                default=0.5,
                help_text="Threshold imposed on dist between two completions: if dist>threshold, they are considered as equal",
                verbose_name="metric_threshold",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="overlap_cohort_percentage",
            field=models.IntegerField(default=100, verbose_name="overlap_cohort_percentage"),
        ),
        migrations.AddField(
            model_name="project",
            name="show_overlap_first",
            field=models.BooleanField(default=True, verbose_name="show overlap first"),
        ),
        migrations.AddField(
            model_name="project",
            name="agreement_threshold",
            field=models.FloatField(
                default=0.0,
                help_text="Minimal agreement score to consider tasks labeled for sending to ML backend (inclusively)",
                verbose_name="agreement_threshold",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="agreement_method",
            field=models.CharField(
                choices=[
                    (
                        "Single linkage",
                        "Threshold based completions grouping using single linkage method",
                    ),
                    (
                        "Complete linkage",
                        "Threshold based completions grouping using complete linkage method",
                    ),
                    (
                        "No grouping",
                        "Compute agreement without grouping (just averaging the distances between labeling results)",
                    ),
                ],
                default="Single linkage",
                max_length=100,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="projecttemplate",
            name="created_by",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="project_templates",
                to=settings.AUTH_USER_MODEL,
                verbose_name="created by",
            ),
        ),
        migrations.AddField(
            model_name="projecttemplate",
            name="is_private",
            field=models.BooleanField(
                default=True,
                help_text="If template is private, it is accessible only from private team",
                verbose_name="private",
            ),
        ),
        migrations.AddField(
            model_name="projecttemplate",
            name="project_settings",
            field=models.JSONField(
                default=dict,
                help_text="general dict serialized project settings",
                verbose_name="project settings",
            ),
        ),
        migrations.AlterField(
            model_name="projecttemplate",
            name="cover_image_url",
            field=models.CharField(
                blank=True, default="", max_length=1000, null=True, verbose_name="cover image"
            ),
        ),
        migrations.AlterField(
            model_name="projecttemplate",
            name="description",
            field=models.TextField(default="", null=True, verbose_name="description"),
        ),
        migrations.AlterField(
            model_name="projecttemplate",
            name="input_example",
            field=models.TextField(blank=True, verbose_name="input example"),
        ),
        migrations.AlterField(
            model_name="projecttemplate",
            name="output_example",
            field=models.TextField(blank=True, verbose_name="output example"),
        ),
        migrations.AddField(
            model_name="project",
            name="control_weights",
            field=models.JSONField(
                default=dict,
                help_text="Weights for control tags",
                null=True,
                verbose_name="control weights",
            ),
        ),
        migrations.CreateModel(
            name="MLBackendPredictionJob",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("job_id", models.CharField(max_length=128)),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="ml_backend_prediction_jobs",
                        to="projects.project",
                    ),
                ),
            ],
        ),
        migrations.DeleteModel(
            name="MLBackendScheduledJob",
        ),
        migrations.AddField(
            model_name="projecttemplate",
            name="organization",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="project_templates",
                to="organizations.organization",
            ),
        ),
        migrations.CreateModel(
            name="ProjectMember",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "enabled",
                    models.BooleanField(default=True, help_text="Project member is enabled"),
                ),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("updated_at", models.DateTimeField(auto_now=True, verbose_name="updated at")),
            ],
        ),
        migrations.AddField(
            model_name="project",
            name="token",
            field=models.CharField(
                blank=True,
                default=core.utils.common.create_hash,
                max_length=256,
                null=True,
                verbose_name="token",
            ),
        ),
        migrations.AddField(
            model_name="projectmember",
            name="project",
            field=models.ForeignKey(
                help_text="Project ID",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="members",
                to="projects.project",
            ),
        ),
        migrations.AddField(
            model_name="projectmember",
            name="user",
            field=models.ForeignKey(
                help_text="User ID",
                on_delete=django.db.models.deletion.CASCADE,
                related_name="project_memberships",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="result_count",
            field=models.IntegerField(
                default=0,
                help_text="Total results inside of completions counter",
                verbose_name="result count",
            ),
        ),
        migrations.CreateModel(
            name="Storage",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                (
                    "title",
                    models.CharField(
                        help_text="Cloud storage title",
                        max_length=256,
                        null=True,
                        verbose_name="title",
                    ),
                ),
                (
                    "description",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage description",
                        null=True,
                        verbose_name="description",
                    ),
                ),
                (
                    "type",
                    models.CharField(
                        choices=[("s3", "AWS S3 storage")], default="s3", max_length=32
                    ),
                ),
                (
                    "path",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage path (e.g. bucket name)",
                        null=True,
                        verbose_name="path",
                    ),
                ),
                (
                    "prefix",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage prefix (e.g. container path)",
                        null=True,
                        verbose_name="prefix",
                    ),
                ),
                (
                    "regex",
                    models.TextField(
                        blank=True,
                        help_text="Cloud storage regex for filtering objects",
                        null=True,
                        verbose_name="regex",
                    ),
                ),
                (
                    "data_key",
                    models.TextField(
                        blank=True,
                        help_text="Data key to connect BLOBs with object tags",
                        null=True,
                        verbose_name="data_key",
                    ),
                ),
                (
                    "use_blob_urls",
                    models.BooleanField(
                        default=False,
                        help_text="Interpret objects as BLOBs and generate URLs",
                        verbose_name="use BLOB URLs",
                    ),
                ),
                (
                    "params",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        help_text="Cloud storage specific parameters (e.g. credentials stored in key: value format",
                        null=True,
                        verbose_name="params",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="storages",
                        to="projects.project",
                    ),
                ),
            ],
        ),
        migrations.AlterField(
            model_name="project",
            name="task_data_login",
            field=models.CharField(
                blank=True,
                help_text="Task data credentials: login",
                max_length=256,
                null=True,
                verbose_name="task_data_login",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="task_data_password",
            field=models.CharField(
                blank=True,
                help_text="Task data credentials: password",
                max_length=256,
                null=True,
                verbose_name="task_data_password",
            ),
        ),
        migrations.AddField(
            model_name="project",
            name="organization",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="projects",
                to="organizations.organization",
            ),
        ),
        migrations.CreateModel(
            name="ProjectSummary",
            fields=[
                (
                    "project",
                    annoying.fields.AutoOneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        primary_key=True,
                        related_name="summary",
                        serialize=False,
                        to="projects.project",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, help_text="Creation time", verbose_name="created at"
                    ),
                ),
                (
                    "all_data_columns",
                    models.JSONField(
                        default=dict,
                        help_text="All data columns found in imported tasks",
                        null=True,
                        verbose_name="all data columns",
                    ),
                ),
                (
                    "common_data_columns",
                    models.JSONField(
                        default=list,
                        help_text="Common data columns found in imported tasks",
                        null=True,
                        verbose_name="common data columns",
                    ),
                ),
                (
                    "created_annotations",
                    models.JSONField(
                        default=dict,
                        help_text="Unique annotation types identified by tuple (from_name, to_name, type)",
                        null=True,
                        verbose_name="created annotations",
                    ),
                ),
                (
                    "created_labels",
                    models.JSONField(
                        default=dict,
                        help_text="Unique labels",
                        null=True,
                        verbose_name="created labels",
                    ),
                ),
            ],
        ),
        migrations.DeleteModel(
            name="UploadedDataFile",
        ),
        migrations.DeleteModel(
            name="Storage",
        ),
        migrations.AddField(
            model_name="project",
            name="is_draft",
            field=models.BooleanField(
                default=False,
                help_text="Whether the project is in the middle of creation process",
                verbose_name="is draft",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="label_config",
            field=models.TextField(
                blank=True,
                default="<View></View>",
                help_text="Label config in XML format. See more about it in documentation",
                null=True,
                verbose_name="label config",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="title",
            field=models.CharField(
                blank=True,
                help_text="Project name, length is from 3 to 50 chars",
                max_length=50,
                null=True,
                validators=[
                    django.core.validators.MinLengthValidator(3),
                    django.core.validators.MaxLengthValidator(50),
                ],
                verbose_name="name",
            ),
        ),
        migrations.RemoveField(
            model_name="project",
            name="batch_size",
        ),
        migrations.RemoveField(
            model_name="project",
            name="cluster_annotation",
        ),
        migrations.RemoveField(
            model_name="project",
            name="has_finished",
        ),
        migrations.RemoveField(
            model_name="project",
            name="onboarding_steps",
        ),
        migrations.RemoveField(
            model_name="project",
            name="skip_onboarding",
        ),
        migrations.AddField(
            model_name="project",
            name="description",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Expert instruction in HTML format",
                null=True,
                verbose_name="expert instruction",
            ),
        ),
        migrations.RemoveField(
            model_name="mlbackendconnection",
            name="ml_backend",
        ),
        migrations.RemoveField(
            model_name="mlbackendconnection",
            name="project",
        ),
        migrations.RemoveField(
            model_name="mlbackendpredictionjob",
            name="project",
        ),
        migrations.RemoveField(
            model_name="mlbackendtrainjob",
            name="project",
        ),
        migrations.RemoveField(
            model_name="projectstats",
            name="project",
        ),
        migrations.RemoveField(
            model_name="project",
            name="ml_backend_active_connection",
        ),
        migrations.RemoveField(
            model_name="projecttemplate",
            name="ml_backend",
        ),
        migrations.DeleteModel(
            name="MLBackend",
        ),
        migrations.DeleteModel(
            name="MLBackendConnection",
        ),
        migrations.DeleteModel(
            name="MLBackendPredictionJob",
        ),
        migrations.DeleteModel(
            name="MLBackendTrainJob",
        ),
        migrations.DeleteModel(
            name="ProjectStats",
        ),
        migrations.AlterField(
            model_name="project",
            name="title",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Project name, length is from 3 to 50 chars",
                max_length=50,
                null=True,
                validators=[
                    django.core.validators.MinLengthValidator(3),
                    django.core.validators.MaxLengthValidator(50),
                ],
                verbose_name="title",
            ),
        ),
    ]



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
from core.settings.base import *

DJANGO_DB = get_env("DJANGO_DB", DJANGO_DB_SQLITE)
DATABASES = {"default": DATABASES_ALL[DJANGO_DB]}

MIDDLEWARE.append("organizations.middleware.DummyGetSessionMiddleware")
MIDDLEWARE.append("core.middleware.UpdateLastActivityMiddleware")

ADD_DEFAULT_ML_BACKENDS = False

LOGGING["root"]["level"] = get_env("LOG_LEVEL", "DEBUG")

DEBUG = get_bool_env("DEBUG", False)

DEBUG_PROPAGATE_EXCEPTIONS = get_bool_env("DEBUG_PROPAGATE_EXCEPTIONS", False)

SESSION_COOKIE_SECURE = False

SESSION_ENGINE = "django.contrib.sessions.backends.signed_cookies"

RQ_QUEUES = {}



# Generated by Django 3.1.4 on 2021-03-06 05:06

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projects", "0003_project_color"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="project",
            name="agreement_method",
        ),
        migrations.RemoveField(
            model_name="project",
            name="agreement_threshold",
        ),
        migrations.RemoveField(
            model_name="project",
            name="metric_name",
        ),
        migrations.RemoveField(
            model_name="project",
            name="metric_params",
        ),
        migrations.RemoveField(
            model_name="project",
            name="metric_threshold",
        ),
        migrations.RemoveField(
            model_name="project",
            name="use_kappa",
        ),
        migrations.AlterField(
            model_name="project",
            name="description",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Project description",
                null=True,
                verbose_name="description",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="enable_empty_completion",
            field=models.BooleanField(
                default=True,
                help_text="Allow annotators to submit empty annotations",
                verbose_name="enable empty annotation",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="expert_instruction",
            field=models.TextField(
                blank=True,
                default="",
                help_text="Labeling instructions in HTML format",
                null=True,
                verbose_name="expert instruction",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="is_draft",
            field=models.BooleanField(
                default=False,
                help_text="Whether or not the project is in the middle of being created",
                verbose_name="is draft",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="is_published",
            field=models.BooleanField(
                default=False,
                help_text="Whether or not the project is published to annotators",
                verbose_name="published",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="maximum_completions",
            field=models.IntegerField(
                default=1,
                help_text="Maximum number of annotations for one task. If the number of annotations per task is equal or greater to this value, the task is completed (is_labeled=True)",
                verbose_name="maximum completion number",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="min_completions_to_start_training",
            field=models.IntegerField(
                default=10,
                help_text="Minimum number of completed tasks after which model training is started",
                verbose_name="min_completions_to_start_training",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="result_count",
            field=models.IntegerField(
                default=0,
                help_text="Total results inside of annotations counter",
                verbose_name="result count",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="show_collab_predictions",
            field=models.BooleanField(
                default=True,
                help_text="If set, the annotator can view model predictions",
                verbose_name="show predictions to annotator",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="show_completion_history",
            field=models.BooleanField(
                default=False,
                help_text="Show annotation history to annotator",
                verbose_name="show annotation history",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="show_instruction",
            field=models.BooleanField(
                default=False,
                help_text="Show instructions to the annotator before they start",
                verbose_name="show instruction",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="show_skip_button",
            field=models.BooleanField(
                default=True,
                help_text="Show a skip button in interface and allow annotators to skip the task",
                verbose_name="show skip button",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="title",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Project name. Must be between 3 to 50 characters long.",
                max_length=50,
                null=True,
                validators=[
                    django.core.validators.MinLengthValidator(3),
                    django.core.validators.MaxLengthValidator(50),
                ],
                verbose_name="title",
            ),
        ),
        migrations.AlterField(
            model_name="projectsummary",
            name="common_data_columns",
            field=models.JSONField(
                default=list,
                help_text="Common data columns found across imported tasks",
                null=True,
                verbose_name="common data columns",
            ),
        ),
        migrations.AlterField(
            model_name="projecttemplate",
            name="expert_instruction",
            field=models.TextField(default="", verbose_name="annotator instructions"),
        ),
    ]



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
from django.shortcuts import redirect, render, reverse
from rules.contrib.views import permission_required, objectgetter

from core.permissions import IsBusiness, get_object_with_permissions, view_with_auth
from core.utils.common import (
    get_object_with_check_and_log,
    find_editor_files,
    get_organization_from_request,
)
from core.version import get_short_version
from organizations.models import Organization
from projects.models import Project


@view_with_auth(["GET"], (IsBusiness,))
# @permission_required('tasks.delete_task', fn=objectgetter(Project, 'pk'), raise_exception=True)
def task_page(request, pk):
    project = get_object_with_check_and_log(request, Project, pk=pk)

    response = {"project": project, "version": get_short_version()}
    response.update(find_editor_files())
    return render(request, "data_manager/data.html", response)



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import logging

from rest_framework import generics
from rest_framework.views import APIView
from core.permissions import all_permissions
from rest_framework.parsers import FormParser, JSONParser, MultiPartParser
from rest_framework.response import Response

from .s3.api import S3ImportStorageListAPI, S3ExportStorageListAPI
from .gcs.api import GCSImportStorageListAPI, GCSExportStorageListAPI
from .azure_blob.api import AzureBlobImportStorageListAPI, AzureBlobExportStorageListAPI
from .redis.api import RedisImportStorageListAPI, RedisExportStorageListAPI
from .localfiles.api import LocalFilesImportStorageListAPI, LocalFilesExportStorageListAPI

logger = logging.getLogger(__name__)
# TODO: replace hardcoded apps lists with search over included storage apps


class AllImportStorageTypesAPI(APIView):
    permission_required = all_permissions.projects_change
    swagger_schema = None

    def get(self, request, **kwargs):
        return Response(
            [
                {"name": "s3", "title": "AWS S3"},
                {"name": "gcs", "title": "Google Cloud Storage"},
                {"name": "azure", "title": "Microsoft Azure"},
                {"name": "redis", "title": "Redis"},
                {"name": "localfiles", "title": "Local files"},
            ]
        )


class AllExportStorageTypesAPI(APIView):
    permission_required = all_permissions.projects_change
    swagger_schema = None

    def get(self, request, **kwargs):
        return Response(
            [
                {"name": "s3", "title": "AWS S3"},
                {"name": "gcs", "title": "Google Cloud Storage"},
                {"name": "azure", "title": "Microsoft Azure"},
                {"name": "redis", "title": "Redis"},
                {"name": "localfiles", "title": "Local files"},
            ]
        )


class AllImportStorageListAPI(generics.ListAPIView):

    parser_classes = (JSONParser, FormParser, MultiPartParser)
    permission_required = all_permissions.projects_change
    swagger_schema = None

    def _get_response(self, api, request, *args, **kwargs):
        try:
            view = api.as_view()
            response = view(request._request, *args, **kwargs)
            payload = response.data
            if not isinstance(payload, list):
                raise ValueError(f"Response is not list")
            return response.data
        except Exception as exc:
            logger.error(f"Can't process {api.__class__.__name__}. Reason: {exc}", exc_info=True)
            return []

    def list(self, request, *args, **kwargs):
        return Response(
            data=sum(
                [
                    self._get_response(S3ImportStorageListAPI, request, *args, **kwargs),
                    self._get_response(GCSImportStorageListAPI, request, *args, **kwargs),
                    self._get_response(AzureBlobImportStorageListAPI, request, *args, **kwargs),
                    self._get_response(RedisImportStorageListAPI, request, *args, **kwargs),
                    self._get_response(LocalFilesImportStorageListAPI, request, *args, **kwargs),
                ],
                [],
            )
        )


class AllExportStorageListAPI(generics.ListAPIView):

    parser_classes = (JSONParser, FormParser, MultiPartParser)
    permission_required = all_permissions.projects_change
    swagger_schema = None

    def _get_response(self, api, request, *args, **kwargs):
        view = api.as_view()
        response = view(request._request, *args, **kwargs)
        return response.data

    def list(self, request, *args, **kwargs):
        return Response(
            data=sum(
                [
                    self._get_response(S3ExportStorageListAPI, request, *args, **kwargs),
                    self._get_response(GCSExportStorageListAPI, request, *args, **kwargs),
                    self._get_response(AzureBlobExportStorageListAPI, request, *args, **kwargs),
                    self._get_response(RedisExportStorageListAPI, request, *args, **kwargs),
                    self._get_response(LocalFilesExportStorageListAPI, request, *args, **kwargs),
                ],
                [],
            )
        )



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import json

from rest_framework import serializers
from django.db import transaction

from data_manager.models import View, Filter, FilterGroup
from tasks.models import Task
from tasks.serializers import (
    TaskSerializer,
    AnnotationSerializer,
    PredictionSerializer,
    AnnotationDraftSerializer,
)


class FilterSerializer(serializers.ModelSerializer):
    class Meta:
        model = Filter
        fields = "__all__"


class FilterGroupSerializer(serializers.ModelSerializer):
    filters = FilterSerializer(many=True)

    class Meta:
        model = FilterGroup
        fields = "__all__"


class ViewSerializer(serializers.ModelSerializer):
    filter_group = FilterGroupSerializer(required=False)

    class Meta:
        model = View
        fields = "__all__"

    def to_internal_value(self, data):
        """
        map old filters structure to models
        "filters": {  ===> FilterGroup model
            "conjunction": "or",
            "items":[  ===> "filters" in FilterGroup
                 {  ==> Filter model
                   "filter":"filter:tasks:data.image", ==> column
                    "operator":"contains",
                    "type":"Image",
                    "value": <string: "XXX" | int: 123 | dict | list>
                 },
                  {
                    "filter":"filter:tasks:data.image",
                    "operator":"equal",
                    "type":"Image",
                    "value": <string: "XXX" | int: 123 | dict | list>
                 }
              ]
           }
        }
        """
        _data = data.get("data", {})

        filters = _data.pop("filters", {})
        conjunction = filters.get("conjunction")
        if "filter_group" not in data and conjunction:
            data["filter_group"] = {"conjunction": conjunction, "filters": []}
            if "items" in filters:
                for f in filters["items"]:
                    data["filter_group"]["filters"].append(
                        {
                            "column": f.get("filter", ""),
                            "operator": f.get("operator", ""),
                            "type": f.get("type", ""),
                            "value": f.get("value", {}),
                        }
                    )

        ordering = _data.pop("ordering", {})
        data["ordering"] = ordering

        return super().to_internal_value(data)

    def to_representation(self, instance):
        result = super().to_representation(instance)
        filters = result.pop("filter_group", {})
        if filters:
            filters["items"] = []
            filters.pop("filters", [])
            filters.pop("id", None)

            for f in instance.filter_group.filters.order_by("index"):
                filters["items"].append(
                    {
                        "filter": f.column,
                        "operator": f.operator,
                        "type": f.type,
                        "value": f.value,
                    }
                )
            result["data"]["filters"] = filters

        selected_items = result.pop("selected_items", {})
        if selected_items:
            result["data"]["selectedItems"] = selected_items

        ordering = result.pop("ordering", {})
        if ordering:
            result["data"]["ordering"] = ordering
        return result

    @staticmethod
    def _create_filters(filter_group, filters_data):
        filter_index = 0
        for filter_data in filters_data:
            filter_data["index"] = filter_index
            filter_group.filters.add(Filter.objects.create(**filter_data))
            filter_index += 1

    def create(self, validated_data):
        with transaction.atomic():
            filter_group_data = validated_data.pop("filter_group", None)
            if filter_group_data:
                filters_data = filter_group_data.pop("filters", [])
                filter_group = FilterGroup.objects.create(**filter_group_data)

                self._create_filters(filter_group=filter_group, filters_data=filters_data)

                validated_data["filter_group_id"] = filter_group.id
            view = View.objects.create(**validated_data)

            return view

    def update(self, instance, validated_data):
        with transaction.atomic():
            filter_group_data = validated_data.pop("filter_group", None)
            if filter_group_data:
                filters_data = filter_group_data.pop("filters", [])

                filter_group = instance.filter_group
                if filter_group is None:
                    filter_group = FilterGroup.objects.create(**filter_group_data)

                conjunction = filter_group_data.get("conjunction")
                if conjunction and filter_group.conjunction != conjunction:
                    filter_group.conjunction = conjunction
                    filter_group.save()

                filter_group.filters.clear()
                self._create_filters(filter_group=filter_group, filters_data=filters_data)

            ordering = validated_data.pop("ordering", None)
            if ordering and ordering != instance.ordering:
                instance.ordering = ordering
                instance.save()

            if validated_data["data"] != instance.data:
                instance.data = validated_data["data"]
                instance.save()

            return instance


class DataManagerTaskSerializer(TaskSerializer):
    predictions = PredictionSerializer(many=True, default=[], read_only=True)
    annotations = AnnotationSerializer(many=True, default=[], read_only=True)
    drafts = serializers.SerializerMethodField()

    cancelled_annotations = serializers.SerializerMethodField()
    completed_at = serializers.SerializerMethodField()
    annotations_results = serializers.SerializerMethodField()
    predictions_results = serializers.SerializerMethodField()
    predictions_score = serializers.SerializerMethodField()
    total_annotations = serializers.SerializerMethodField()
    total_predictions = serializers.SerializerMethodField()
    file_upload = serializers.ReadOnlyField(source="file_upload_name")
    annotators = serializers.SerializerMethodField()

    class Meta:
        model = Task
        ref_name = "data_manager_task_serializer"

        fields = [
            "cancelled_annotations",
            "completed_at",
            "created_at",
            "annotations_results",
            "data",
            "id",
            "predictions_results",
            "predictions_score",
            "total_annotations",
            "total_predictions",
            "annotations",
            "predictions",
            "drafts",
            "file_upload",
            "annotators",
            "project",
        ]

    @staticmethod
    def get_cancelled_annotations(obj):
        return obj.annotations.filter(was_cancelled=True).count()

    @staticmethod
    def get_completed_at(obj):
        annotations = obj.annotations.all()
        if annotations:
            return max(c.created_at for c in annotations)
        return None

    @staticmethod
    def get_annotations_results(obj):
        annotations = obj.annotations.all()
        if annotations:
            return json.dumps([item.result for item in annotations])
        else:
            return ""

    @staticmethod
    def get_predictions_results(obj):
        predictions = obj.predictions.all()
        if predictions:
            return json.dumps([item.result for item in predictions])
        else:
            return ""

    @staticmethod
    def get_predictions_score(obj):
        predictions = obj.predictions.all()
        if predictions:
            values = [item.score for item in predictions if isinstance(item.score, (float, int))]
            if values:
                return sum(values) / float(len(values))
        return None

    @staticmethod
    def get_total_predictions(obj):
        return obj.predictions.count()

    @staticmethod
    def get_total_annotations(obj):
        return obj.annotations.filter(was_cancelled=False).count()

    @staticmethod
    def get_annotators(obj):
        result = obj.annotations.values_list("completed_by", flat=True).distinct()
        result = [r for r in result if r is not None]
        return result

    def get_drafts(self, task):
        """Return drafts only for the current user"""
        # it's for swagger documentation
        if not isinstance(task, Task):
            return AnnotationDraftSerializer(many=True)

        drafts = task.drafts
        if "request" in self.context and hasattr(self.context["request"], "user"):
            user = self.context["request"].user
            drafts = drafts.filter(user=user)

        return AnnotationDraftSerializer(
            drafts, many=True, read_only=True, default=True, context=self.context
        ).data


class SelectedItemsSerializer(serializers.Serializer):
    all = serializers.BooleanField()
    included = serializers.ListField(child=serializers.IntegerField(), required=False)
    excluded = serializers.ListField(child=serializers.IntegerField(), required=False)

    def validate(self, data):
        if data["all"] is True and data.get("included"):
            raise serializers.ValidationError("included not allowed with all==true")
        if data["all"] is False and data.get("excluded"):
            raise serializers.ValidationError("excluded not allowed with all==false")

        view = self.context.get("view")
        request = self.context.get("request")
        if view and request and request.method in ("PATCH", "DELETE"):
            all_value = view.selected_items.get("all")
            if all_value and all_value != data["all"]:
                raise serializers.ValidationError(
                    "changing all value possible only with POST method"
                )

        return data



# Generated by Django 3.1.4 on 2021-03-14 18:40

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projects", "0007_auto_20210309_1304"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="sampling",
            field=models.CharField(
                choices=[
                    ("Sequential sampling", "Tasks are ordered by their IDs"),
                    ("Uniform sampling", "Tasks are chosen randomly"),
                    (
                        "Uncertainty sampling",
                        "Tasks are chosen according to model uncertainty scores (active learning mode)",
                    ),
                ],
                default="Sequential sampling",
                max_length=100,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="title",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Project name. Must be between 3 and 50 characters long.",
                max_length=50,
                null=True,
                validators=[
                    django.core.validators.MinLengthValidator(3),
                    django.core.validators.MaxLengthValidator(50),
                ],
                verbose_name="title",
            ),
        ),
    ]



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import redis
import logging

from django_rq import get_connection

logger = logging.getLogger(__name__)

try:
    _redis = get_connection()
    _redis.ping()
except:
    logger.warning("Redis is not connected.")
    _redis = None


def redis_healthcheck():
    if not _redis:
        return False
    try:
        _redis.ping()
    except redis.exceptions.ConnectionError as exc:
        logger.error(f"Redis healthcheck failed with ConnectionError: {exc}", exc_info=True)
        return False
    except redis.exceptions.TimeoutError as exc:
        logger.error(f"Redis healthcheck failed with TimeoutError: {exc}", exc_info=True)
        return False
    except redis.exceptions.RedisError as exc:
        logger.error(f"Redis healthcheck failed: {exc}", exc_info=True)
        return False
    else:
        logger.debug("Redis client is alive!")
        return True


def redis_connected():
    return redis_healthcheck()


def redis_get(key):
    if not redis_healthcheck():
        return
    return _redis.get(key)


def redis_hget(key1, key2):
    if not redis_healthcheck():
        return
    return _redis.hget(key1, key2)


def redis_set(key, value, ttl=None):
    if not redis_healthcheck():
        return
    return _redis.set(key, value, ex=ttl)


def redis_hset(key1, key2, value):
    if not redis_healthcheck():
        return
    return _redis.hset(key1, key2, value)


def redis_delete(key):
    if not redis_healthcheck():
        return
    return _redis.delete(key)



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.models import Group
from ml.models import MLBackend
from ml.models import MLBackendTrainJob
from organizations.models import Organization
from projects.models import Project
from tasks.models import Annotation
from tasks.models import Task
from users.models import User


class UserAdminShort(UserAdmin):
    def __init__(self, *args, **kwargs):
        super(UserAdminShort, self).__init__(*args, **kwargs)

        # we have empty username - remove it to escape confuse about empty fields in admin web
        # self.list_display = [l for l in self.list_display if l != 'username']
        self.list_display = ["email", "username", "password"]

        self.fieldsets = (
            (None, {"fields": ("password",)}),
            ("Personal info", {"fields": ("first_name", "last_name", "email")}),
            (
                "Permissions",
                {
                    "fields": (
                        "is_active",
                        "is_staff",
                        "is_superuser",
                    )
                },
            ),
            ("Important dates", {"fields": ("last_login", "date_joined")}),
        )

        self.add_fieldsets = (
            (
                None,
                {
                    "classes": ("wide",),
                    "fields": ("email", "username", "password1", "password2"),
                },
            ),
        )


admin.site.register(User, UserAdminShort)
admin.site.register(Project)
admin.site.register(MLBackend)
admin.site.register(MLBackendTrainJob)
admin.site.register(Task)
admin.site.register(Annotation)
admin.site.register(Organization)
# remove unused django groups
admin.site.unregister(Group)



# Generated by Django 3.1.4 on 2021-03-08 15:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("projects", "0005_merge_20210308_1141"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="enable_empty_annotation",
            field=models.BooleanField(
                default=True,
                help_text="Allow annotators to submit empty annotations",
                verbose_name="enable empty annotation",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="maximum_annotations",
            field=models.IntegerField(
                default=1,
                help_text="Maximum number of annotations for one task. If the number of annotations per task is equal or greater to this value, the task is completed (is_labeled=True)",
                verbose_name="maximum annotation number",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="min_annotations_to_start_training",
            field=models.IntegerField(
                default=10,
                help_text="Minimum number of completed tasks after which model training is started",
                verbose_name="min_annotations_to_start_training",
            ),
        ),
        migrations.AlterField(
            model_name="project",
            name="show_annotation_history",
            field=models.BooleanField(
                default=False,
                help_text="Show annotation history to annotator",
                verbose_name="show annotation history",
            ),
        ),
    ]



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
from django.utils.decorators import method_decorator
from drf_yasg.utils import swagger_auto_schema
from io_storages.localfiles.models import LocalFilesImportStorage, LocalFilesExportStorage
from io_storages.localfiles.serializers import (
    LocalFilesImportStorageSerializer,
    LocalFilesExportStorageSerializer,
)
from io_storages.api import (
    ImportStorageListAPI,
    ImportStorageDetailAPI,
    ImportStorageSyncAPI,
    ExportStorageListAPI,
    ExportStorageDetailAPI,
    ImportStorageValidateAPI,
    ExportStorageValidateAPI,
    ImportStorageFormLayoutAPI,
    ExportStorageFormLayoutAPI,
)


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Get all import storage",
        operation_description="Get a list of all local file import storage connections.",
    ),
)
@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Create import storage",
        operation_description="Create a new local file import storage connection.",
    ),
)
class LocalFilesImportStorageListAPI(ImportStorageListAPI):
    queryset = LocalFilesImportStorage.objects.all()
    serializer_class = LocalFilesImportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Get import storage",
        operation_description="Get a specific local file import storage connection.",
    ),
)
@method_decorator(
    name="patch",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Update import storage",
        operation_description="Update a specific local file import storage connection.",
    ),
)
@method_decorator(
    name="delete",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Delete import storage",
        operation_description="Delete a specific local import storage connection.",
    ),
)
class LocalFilesImportStorageDetailAPI(ImportStorageDetailAPI):
    queryset = LocalFilesImportStorage.objects.all()
    serializer_class = LocalFilesImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Sync import storage",
        operation_description="Sync tasks from a local file import storage connection.",
    ),
)
class LocalFilesImportStorageSyncAPI(ImportStorageSyncAPI):
    serializer_class = LocalFilesImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Validate import storage",
        operation_description="Validate a specific local file import storage connection.",
    ),
)
class LocalFilesImportStorageValidateAPI(ImportStorageValidateAPI):
    serializer_class = LocalFilesImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Validate export storage",
        operation_description="Validate a specific local file export storage connection.",
    ),
)
class LocalFilesExportStorageValidateAPI(ExportStorageValidateAPI):
    serializer_class = LocalFilesExportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Get all export storage",
        operation_description="Get a list of all Local export storage connections.",
    ),
)
@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Create export storage",
        operation_description="Create a new local file export storage connection to store annotations.",
    ),
)
class LocalFilesExportStorageListAPI(ExportStorageListAPI):
    queryset = LocalFilesExportStorage.objects.all()
    serializer_class = LocalFilesExportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Get export storage",
        operation_description="Get a specific local file export storage connection.",
    ),
)
@method_decorator(
    name="patch",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Update export storage",
        operation_description="Update a specific local file export storage connection.",
    ),
)
@method_decorator(
    name="delete",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="Local: Delete export storage",
        operation_description="Delete a specific local file export storage connection.",
    ),
)
class LocalFilesExportStorageDetailAPI(ExportStorageDetailAPI):
    queryset = LocalFilesExportStorage.objects.all()
    serializer_class = LocalFilesExportStorageSerializer


class LocalFilesImportStorageFormLayoutAPI(ImportStorageFormLayoutAPI):
    pass


class LocalFilesExportStorageFormLayoutAPI(ExportStorageFormLayoutAPI):
    pass



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
import pytest
import json
import copy

from django.db import transaction
from tasks.models import Task, Annotation, Prediction
from tasks.serializers import AnnotationSerializer
from django.apps import apps


@pytest.mark.skip(reason="HTX-868")
@pytest.mark.parametrize(
    "annotation_items, aggregated_class",
    [
        (
            [
                {
                    "id": 1,
                    "review_result": None,
                    "ground_truth": False,
                    "result": [
                        {
                            "id": "123",
                            "type": "choices",
                            "value": {"choices": ["class_AA"]},
                            "to_name": "text",
                            "from_name": "text_class",
                        }
                    ],
                    "created_at": "test",
                    "updated_at": "test",
                    "lead_time": None,
                    "completed_by": 2,
                }
            ],
            "class_AA",
        ),
        (
            [
                {
                    "id": 2,
                    "review_result": None,
                    "ground_truth": False,
                    "result": [
                        {
                            "id": "123",
                            "type": "choices",
                            "value": {"choices": ["class_AA"]},
                            "to_name": "text",
                            "from_name": "text_class",
                        }
                    ],
                    "created_at": "",
                    "updated_at": "",
                    "lead_time": None,
                    "completed_by": 4,
                },
                {
                    "id": 3,
                    "review_result": None,
                    "ground_truth": False,
                    "result": [
                        {
                            "id": "456",
                            "type": "choices",
                            "value": {"choices": ["class_AA"]},
                            "to_name": "text",
                            "from_name": "text_class",
                        }
                    ],
                    "created_at": "",
                    "updated_at": "",
                    "lead_time": None,
                    "completed_by": 4,
                },
            ],
            "class_AA",
        ),
        (
            [
                {
                    "id": 4,
                    "review_result": None,
                    "ground_truth": False,
                    "result": [
                        {
                            "id": "123",
                            "type": "choices",
                            "value": {"choices": ["class_AA"]},
                            "to_name": "text",
                            "from_name": "text_class",
                        }
                    ],
                    "created_at": "",
                    "updated_at": "",
                    "lead_time": None,
                    "completed_by": 6,
                },
                {
                    "id": 5,
                    "review_result": None,
                    "ground_truth": False,
                    "result": [
                        {
                            "id": "456",
                            "type": "choices",
                            "value": {"choices": ["class_BB"]},
                            "to_name": "text",
                            "from_name": "text_class",
                        }
                    ],
                    "created_at": "",
                    "updated_at": "",
                    "lead_time": None,
                    "completed_by": 6,
                },
                {
                    "id": 6,
                    "review_result": None,
                    "ground_truth": False,
                    "result": [
                        {
                            "id": "789",
                            "type": "choices",
                            "value": {"choices": ["class_BB"]},
                            "to_name": "text",
                            "from_name": "text_class",
                        }
                    ],
                    "created_at": "",
                    "updated_at": "",
                    "lead_time": None,
                    "completed_by": 6,
                },
            ],
            "class_BB",
        ),
        (
            [
                {
                    "id": 7,
                    "review_result": None,
                    "ground_truth": False,
                    "result": [
                        {
                            "id": "123",
                            "type": "choices",
                            "value": {"choices": ["class_AA"]},
                            "to_name": "text",
                            "from_name": "text_class",
                        }
                    ],
                    "created_at": "",
                    "updated_at": "",
                    "lead_time": None,
                    "completed_by": 10,
                }
            ],
            "class_AA",
        ),
    ],
)
@pytest.mark.parametrize(
    "finished, aggregator_type, return_task, num_task_in_result",
    [
        ("0", "no_aggregation", "0", 2),
        ("1", "no_aggregation", "0", 1),
        ("0", "majority_vote", "0", 2),
        ("0", "majority_vote", "1", 2),
        ("1", "majority_vote", "1", 1),
    ],
)
@pytest.mark.django_db
def test_export(
    business_client,
    configured_project,
    finished,
    aggregator_type,
    return_task,
    num_task_in_result,
    annotation_items,
    aggregated_class,
):
    if aggregator_type == "majority_vote" and not apps.is_installed("businesses"):
        pytest.skip("Not supported aggregation for open-source version")

    task_query = Task.objects.filter(project=configured_project.id)
    task = task_query.first()

    expected_annotations_for_task = set()
    for annotation in annotation_items:
        db_annotation = Annotation.objects.create(
            task=task, result=annotation["result"], completed_by=business_client.admin
        )
        db_annotation = AnnotationSerializer(db_annotation).data
        annotation["id"] = db_annotation["id"]
        annotation["created_at"] = db_annotation["created_at"]
        annotation["updated_at"] = db_annotation["updated_at"]
        annotation["completed_by"] = business_client.admin.id
        expected_annotations_for_task.add(json.dumps(annotation))

    r = business_client.get(
        f"/api/projects/{configured_project.id}/results/",
        data={"finished": finished, "aggregator_type": aggregator_type, "return_task": return_task},
    )
    assert r.status_code == 200
    exports = r.json()

    # test expected number of objects returned
    assert len(exports) == num_task_in_result

    # test whether "id" or full task included in results
    if return_task == "0":
        task_with_annotation = next((t for t in exports if t["id"] == task.id))
        assert task_with_annotation["id"] == task.id
    elif return_task == "1":
        task_with_annotation = next((t for t in exports if t["id"] == task.id))
        assert task_with_annotation["data"] == task.data
    else:
        raise Exception("Incorrect return_task param in test: " + str(return_task))

    # test how aggregation affects annotations
    if aggregator_type == "no_aggregation":
        exported_annotations = set()
        for annotation in task_with_annotation["annotations"]:
            exported_annotations.add(json.dumps(annotation))
        assert exported_annotations == expected_annotations_for_task
        if finished != "1":
            # we expect to see all tasks in exports...
            assert len(exports) == task_query.count()
            # ...as well as task without annotations (with empty results)
            assert all(len(t["annotations"]) == 0 for t in exports if t["id"] != task.id)
    else:
        assert (
            task_with_annotation["annotations"][0]["result"][0]["value"]["choices"][0]
            == aggregated_class
        )


@pytest.mark.skip(reason="HTX-868")
@pytest.mark.parametrize("finished", ("0", "1"))
@pytest.mark.parametrize("return_task", ("0", "1"))
@pytest.mark.parametrize("aggregator_type", ("no_aggregation", "majority_vote"))
@pytest.mark.parametrize(
    "annotation_results, predictions",
    [
        (
            [
                [
                    {
                        "id": "123",
                        "from_name": "text_class",
                        "to_name": "text",
                        "type": "choices",
                        "value": {"choices": ["class_A"]},
                    }
                ]
            ],
            {
                "result": [
                    {
                        "id": "123",
                        "from_name": "text_class",
                        "to_name": "text",
                        "type": "choices",
                        "value": {"choices": ["class_A"]},
                    }
                ],
                "score": 0.5,
            },
        ),
        (
            [
                [
                    {
                        "id": "123",
                        "from_name": "text_class",
                        "to_name": "text",
                        "type": "choices",
                        "value": {"choices": ["class_A"]},
                    }
                ]
            ],
            None,
        ),
    ],
)
@pytest.mark.django_db
def test_export_with_predictions(
    business_client,
    configured_project,
    finished,
    return_task,
    aggregator_type,
    annotation_results,
    predictions,
):
    if aggregator_type == "majority_vote" and not apps.is_installed("businesses"):
        pytest.skip("Not supported aggregation for open-source version")

    tasks = Task.objects.filter(project=configured_project.id)
    task = tasks.first()
    for result in annotation_results:
        for r in result:
            r["completed_by"] = [business_client.admin.id]
        Annotation.objects.create(task=task, result=result, completed_by=business_client.admin)
    if predictions:
        for task in tasks:
            Prediction.objects.create(
                task=task, result=predictions["result"], score=predictions["score"]
            )

    r = business_client.get(
        f"/api/projects/{configured_project.id}/results/",
        data={
            "finished": finished,
            "aggregator_type": aggregator_type,
            "return_task": return_task,
            "return_predictions": "1",
        },
    )
    assert r.status_code == 200
    exports = r.json()
    for task in exports:
        if predictions:
            assert task["predictions"][0]["result"] == predictions["result"]
            assert task["predictions"][0]["score"] == predictions["score"]
        else:
            assert task["predictions"] == []



"""This file and its contents are licensed under the Apache License 2.0. Please see the included NOTICE for copyright information and LICENSE for a copy of the license.
"""
from django.utils.decorators import method_decorator
from drf_yasg.utils import swagger_auto_schema
from io_storages.gcs.models import GCSImportStorage, GCSExportStorage
from io_storages.gcs.serializers import GCSImportStorageSerializer, GCSExportStorageSerializer
from io_storages.api import (
    ImportStorageListAPI,
    ImportStorageDetailAPI,
    ImportStorageSyncAPI,
    ExportStorageListAPI,
    ExportStorageDetailAPI,
    ImportStorageValidateAPI,
    ExportStorageValidateAPI,
    ImportStorageFormLayoutAPI,
    ExportStorageFormLayoutAPI,
)


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Get all import storage",
        operation_description="Get a list of all GCS import storage connections.",
    ),
)
@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Create import storage",
        operation_description="Create a new GCS import storage connection.",
    ),
)
class GCSImportStorageListAPI(ImportStorageListAPI):
    queryset = GCSImportStorage.objects.all()
    serializer_class = GCSImportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Get import storage",
        operation_description="Get a specific GCS import storage connection.",
    ),
)
@method_decorator(
    name="patch",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Update import storage",
        operation_description="Update a specific GCS import storage connection.",
    ),
)
@method_decorator(
    name="delete",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Delete import storage",
        operation_description="Delete a specific GCS import storage connection.",
    ),
)
class GCSImportStorageDetailAPI(ImportStorageDetailAPI):
    queryset = GCSImportStorage.objects.all()
    serializer_class = GCSImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Sync import storage",
        operation_description="Sync tasks from an GCS import storage connection.",
    ),
)
class GCSImportStorageSyncAPI(ImportStorageSyncAPI):
    serializer_class = GCSImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Validate import storage",
        operation_description="Validate a specific GCS import storage connection.",
    ),
)
class GCSImportStorageValidateAPI(ImportStorageValidateAPI):
    serializer_class = GCSImportStorageSerializer


@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Validate export storage",
        operation_description="Validate a specific GCS export storage connection.",
    ),
)
class GCSExportStorageValidateAPI(ExportStorageValidateAPI):
    serializer_class = GCSExportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Get all export storage",
        operation_description="Get a list of all GCS export storage connections.",
    ),
)
@method_decorator(
    name="post",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Create export storage",
        operation_description="Create a new GCS export storage connection to store annotations.",
    ),
)
class GCSExportStorageListAPI(ExportStorageListAPI):
    queryset = GCSExportStorage.objects.all()
    serializer_class = GCSExportStorageSerializer


@method_decorator(
    name="get",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Get export storage",
        operation_description="Get a specific GCS export storage connection.",
    ),
)
@method_decorator(
    name="patch",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Update export storage",
        operation_description="Update a specific GCS export storage connection.",
    ),
)
@method_decorator(
    name="delete",
    decorator=swagger_auto_schema(
        tags=["Storage"],
        operation_summary="GCS: Delete export storage",
        operation_description="Delete a specific GCS export storage connection.",
    ),
)
class GCSExportStorageDetailAPI(ExportStorageDetailAPI):
    queryset = GCSExportStorage.objects.all()
    serializer_class = GCSExportStorageSerializer


class GCSImportStorageFormLayoutAPI(ImportStorageFormLayoutAPI):
    pass


class GCSExportStorageFormLayoutAPI(ExportStorageFormLayoutAPI):
    pass
